(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     78994,       2044]
NotebookOptionsPosition[     77173,       1978]
NotebookOutlinePosition[     77529,       1994]
CellTagsIndexPosition[     77486,       1991]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Utils", "Section",
 CellChangeTimes->{{3.5067964703290577`*^9, 3.50679647294584*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"graphCoords", "[", "g_Graph", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{"Thread", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"VertexList", "[", "g", "]"}], "->", "VertexCoordinates"}], "/.", 
       RowBox[{"AbsoluteOptions", "[", "g", "]"}]}], "]"}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"twoColorGraph", "[", 
     RowBox[{"g_", ",", "seen_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"vcoords", "=", 
       RowBox[{"graphCoords", "[", "g", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"verts", "=", 
       RowBox[{"VertexList", "[", "g", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"edges", "=", 
       RowBox[{"EdgeList", "[", "g", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"frag$vsf", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"x_", ",", "y_"}], "}"}], ",", "v_", ",", 
         RowBox[{"{", 
          RowBox[{"w_", ",", "h_"}], "}"}]}], "]"}], ":=", 
       RowBox[{"{", 
        RowBox[{"Black", ",", 
         RowBox[{"EdgeForm", "[", "Thick", "]"}], ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"MemberQ", "[", 
            RowBox[{"seen", ",", "v"}], "]"}], ",", "Pink", ",", "White"}], 
          "]"}], ",", 
         RowBox[{"Disk", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"x", ",", "y"}], "}"}], ",", 
           RowBox[{"1", "/", "4"}]}], "]"}]}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Graph", "[", 
       RowBox[{"verts", ",", "edges", ",", 
        RowBox[{"VertexShapeFunction", "\[Rule]", "frag$vsf"}], ",", 
        RowBox[{"VertexCoordinates", "\[Rule]", "vcoords"}], ",", 
        RowBox[{"ImageSize", "\[Rule]", "60"}]}], "]"}]}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"initGraph", "[", "gg_Graph", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"g", "=", "gg"}], ";", "\[IndentingNewLine]", 
      RowBox[{"lineThickness", "=", 
       RowBox[{"1", "/", "2"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"masterScale", "=", "100"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"jnodes", ",", "jedges"}], "}"}], "=", 
       RowBox[{"findTreeDecomposition", "[", 
        RowBox[{"g", ",", 
         RowBox[{"PrimaryHeuristic", "\[Rule]", "\"\<minfill\>\""}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"gap", "=", "lineThickness"}], ";", "\[IndentingNewLine]", 
      RowBox[{"colorList", "=", 
       RowBox[{"ColorData", "[", 
        RowBox[{"\"\<Crayola\>\"", ",", "\"\<ColorList\>\""}], "]"}]}], ";"}],
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"checkClusterDecomposition", "[", 
     RowBox[{"g_", ",", 
      RowBox[{"{", 
       RowBox[{"jnodes_", ",", "jedges_"}], "}"}]}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"isok", "=", "True"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"existNodes", "=", 
       RowBox[{"Union", "[", 
        RowBox[{"VertexList", "[", "g", "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"coveredNodes", "=", 
       RowBox[{"Union", "[", 
        RowBox[{"Flatten", "[", "jnodes", "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"prop", "=", 
       RowBox[{"(", 
        RowBox[{"existNodes", "===", "coveredNodes"}], ")"}]}], ";", "\n", 
      "\t", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", "prop"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<checkClusterDecomposition: violates variable preservation for \
\>\"", ",", 
           RowBox[{"existNodes", "\[Backslash]", "coveredNodes"}]}], "]"}], 
         ";", 
         RowBox[{"isok", "=", "False"}]}]}], "]"}], ";", "\n", 
      "\[IndentingNewLine]", 
      RowBox[{"existFactors", "=", 
       RowBox[{"deepSort", "@", 
        RowBox[{"Union", "[", 
         RowBox[{"List", "@@@", 
          RowBox[{"EdgeList", "[", "g", "]"}]}], "]"}]}]}], ";", 
      RowBox[{"coveredFactors", "=", 
       RowBox[{"Union", "[", 
        RowBox[{"Select", "[", 
         RowBox[{"existFactors", ",", 
          RowBox[{
           RowBox[{"anyTrue", "[", 
            RowBox[{
             RowBox[{"bag", "\[Element]", "jnodes"}], ",", 
             RowBox[{"#", "\[Subset]", "bag"}]}], "]"}], "&"}]}], "]"}], 
        "]"}]}], ";", 
      RowBox[{"prop", "=", 
       RowBox[{"(", 
        RowBox[{"existFactors", "===", "coveredFactors"}], ")"}]}], ";", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", "prop"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<checkClusterDecomposition: violates factor preservation for \>\
\"", ",", 
           RowBox[{"existFactors", "\[Backslash]", "coveredFactors"}]}], 
          "]"}], ";", 
         RowBox[{"isok", "=", "False"}]}]}], "]"}], ";", "\n", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
       "*", " ", "maybe", " ", "check", " ", "separator", " ", "properties", 
        " ", "too", " ", "later"}], " ", "**)"}], "\[IndentingNewLine]", 
      "isok"}], "\[IndentingNewLine]", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"drawFragment", "[", 
     RowBox[{"nodes_", ",", "scale_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"OrderedQ", "[", "nodes", "]"}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"points", "=", 
       RowBox[{"nodes", "/.", "vcoords"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"plotRange", "=", 
       RowBox[{"getPlotRange", "[", "points", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"imageSize", "=", 
       RowBox[{
        RowBox[{"-", "scale"}], "*", 
        RowBox[{"Subtract", "@@@", "plotRange"}]}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"Clear", "[", "frag$vsf", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"frag$vsf", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"x_", ",", "y_"}], "}"}], ",", "v_", ",", 
         RowBox[{"{", 
          RowBox[{"w_", ",", "h_"}], "}"}]}], "]"}], ":=", 
       RowBox[{"{", 
        RowBox[{"Black", ",", 
         RowBox[{"Circle", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"x", ",", "y"}], "}"}], ",", 
           RowBox[{"lineThickness", "/", "2"}]}], "]"}], ",", 
         RowBox[{"Text", "[", 
          RowBox[{"v", ",", 
           RowBox[{"{", 
            RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], "}"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"hull", "=", 
       RowBox[{"ComputationalGeometry`ConvexHull", "[", 
        RowBox[{"points", ",", 
         RowBox[{"AllPoints", "\[Rule]", "False"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"coverP", "=", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"colorList", "\[LeftDoubleBracket]", 
            RowBox[{"idx", "[", 
             RowBox[{"jnodes", ",", "nodes"}], "]"}], 
            "\[RightDoubleBracket]"}], ",", 
           RowBox[{"AbsoluteThickness", "[", 
            RowBox[{"scale", "*", "lineThickness"}], "]"}], ",", 
           RowBox[{"JoinForm", "[", "\"\<Round\>\"", "]"}], ",", 
           RowBox[{"JoinedCurve", "[", 
            RowBox[{
             RowBox[{"Line", "[", 
              RowBox[{"points", "[", 
               RowBox[{"[", "hull", "]"}], "]"}], "]"}], ",", 
             RowBox[{"CurveClosed", "\[Rule]", "True"}]}], "]"}]}], "}"}], 
         ",", 
         RowBox[{"PlotRange", "\[Rule]", "plotRange"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "imageSize"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"graphP", "=", 
       RowBox[{"Show", "[", 
        RowBox[{"Subgraph", "[", 
         RowBox[{"g", ",", "nodes", ",", 
          RowBox[{"VertexCoordinates", "\[Rule]", "vcoords"}], ",", 
          RowBox[{"VertexShapeFunction", "\[Rule]", "frag$vsf"}], ",", 
          RowBox[{"PlotRange", "\[Rule]", "plotRange"}], ",", 
          RowBox[{"ImageSize", "\[Rule]", "imageSize"}]}], "]"}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Show", "[", 
       RowBox[{"coverP", ",", "graphP"}], "]"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"makeClusterGraph", "[", 
    RowBox[{"g_Graph", ",", 
     RowBox[{"{", 
      RowBox[{"jnodes_", ",", "jedges_"}], "}"}], ",", 
     RowBox[{"gap_:", 
      RowBox[{"1", "/", "2"}]}]}], "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Assert", "[", 
      RowBox[{"checkClusterDecomposition", "[", 
       RowBox[{"g", ",", 
        RowBox[{"{", 
         RowBox[{"jnodes", ",", "jedges"}], "}"}]}], "]"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"method", "=", "\"\<minimize\>\""}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"vcoords", "=", 
      RowBox[{"Thread", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"VertexList", "[", "g", "]"}], "->", "VertexCoordinates"}], "/.", 
        RowBox[{"AbsoluteOptions", "[", "g", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"meanBagCoord", "[", "bag_", "]"}], ":=", 
      RowBox[{"Mean", "[", 
       RowBox[{"bag", "/.", "vcoords"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"mcoords", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "\[Rule]", 
         RowBox[{"meanBagCoord", "[", "#", "]"}]}], "&"}], "/@", "jnodes"}]}],
      ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"Switch", "[", 
      RowBox[{
      "method", ",", "\[IndentingNewLine]", "\"\<independentSet\>\"", ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"fixedClusters", "=", 
         RowBox[{"independentClusterSet", "[", "jnodes", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"fcoords", "=", 
         RowBox[{"Select", "[", 
          RowBox[{"mcoords", ",", 
           RowBox[{
            RowBox[{"MemberQ", "[", 
             RowBox[{"fixedClusters", ",", 
              RowBox[{"First", "[", "#", "]"}]}], "]"}], "&"}]}], "]"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"tempPlot", "=", 
         RowBox[{"GraphPlot", "[", 
          RowBox[{
           RowBox[{"Rule", "@@@", "jedges"}], ",", 
           RowBox[{"VertexCoordinateRules", "\[Rule]", "fcoords"}], ",", 
           RowBox[{"VertexLabeling", "\[Rule]", "True"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"mcoords2", "=", 
         RowBox[{"getLabelCoordinateMap", "[", "tempPlot", "]"}]}]}], ",", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", "\"\<minimize\>\"", ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "distance", " ", "of", " ", "cluster", " ", "from", " ", "its", " ", 
         "average", " ", "position"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"objCl", "[", "cluster_", "]"}], ":=", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"k", "=", 
            RowBox[{"idx", "[", 
             RowBox[{"jnodes", ",", "cluster"}], "]"}]}], ";", 
           RowBox[{"Total", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"cluster", "/.", "mcoords"}], ")"}], "-", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"xcl", "[", "k", "]"}], ",", 
                 RowBox[{"ycl", "[", "k", "]"}]}], "}"}]}], ")"}], "^", "2"}],
             "]"}]}], ")"}]}], ";", "\[IndentingNewLine]", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "constrain", " ", "two", " ", "cluster", " ", "to", " ", "be", " ", 
          "at", " ", "a", " ", "distance", " ", "from", " ", "each", " ", 
          "other"}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"consCl", "[", 
          RowBox[{"{", 
           RowBox[{"bag1_", ",", "bag2_"}], "}"}], "]"}], ":=", 
         RowBox[{"(", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"k1", "=", 
            RowBox[{"idx", "[", 
             RowBox[{"jnodes", ",", "bag1"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"k2", "=", 
            RowBox[{"idx", "[", 
             RowBox[{"jnodes", ",", "bag2"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"xcl", "[", "k1", "]"}], "-", 
                RowBox[{"xcl", "[", "k2", "]"}]}], ")"}], "^", "2"}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"ycl", "[", "k1", "]"}], "-", 
                RowBox[{"ycl", "[", "k2", "]"}]}], ")"}], "^", "2"}]}], 
            "\[GreaterEqual]", "gap"}]}], "\[IndentingNewLine]", ")"}]}], ";",
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"obj", "=", 
         RowBox[{"Total", "[", 
          RowBox[{"objCl", "/@", "jnodes"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"cons", "=", 
           RowBox[{"consCl", "/@", "jedges"}]}], ";"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"cons", "=", 
         RowBox[{"consCl", "/@", 
          RowBox[{"Select", "[", 
           RowBox[{
            RowBox[{"fl1", "@", 
             RowBox[{"Outer", "[", 
              RowBox[{"List", ",", "jnodes", ",", "jnodes", ",", "1"}], 
              "]"}]}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"First", "@", "#"}], "\[NotEqual]", 
              RowBox[{"Last", "@", "#"}]}], "&"}]}], "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Clear", "[", 
         RowBox[{"xcl", ",", "ycl"}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"varlist", "=", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"xcl", "[", "#", "]"}], ",", 
             RowBox[{"ycl", "[", "#", "]"}]}], "}"}], "&"}], "/@", 
          RowBox[{"Range", "[", 
           RowBox[{"Length", "@", "jnodes"}], "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"vars", "=", 
         RowBox[{"Flatten", "@", "varlist"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"sol", "=", 
         RowBox[{"Minimize", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"obj", ",", "cons"}], "}"}], ",", "vars"}], "]"}]}], ";", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"tempC", "=", 
         RowBox[{"varlist", "/.", 
          RowBox[{"Last", "@", 
           RowBox[{"FindMinimum", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"obj", ",", "cons"}], "}"}], ",", "vars"}], "]"}]}]}]}],
         ";", "\[IndentingNewLine]", 
        RowBox[{"mcoords2", "=", 
         RowBox[{"Thread", "[", 
          RowBox[{"jnodes", "\[Rule]", "tempC"}], "]"}]}]}], ",", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Assert", "[", "\"\<Unknown arrangement method\>\"", "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"Clear", "[", "vsf", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"vsf", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x_", ",", "y_"}], "}"}], ",", "v_", ",", 
        RowBox[{"{", 
         RowBox[{"w_", ",", "h_"}], "}"}]}], "]"}], ":=", 
      RowBox[{"Inset", "[", 
       RowBox[{
        RowBox[{"drawFragment", "[", 
         RowBox[{"v", ",", "30"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Graph", "[", 
      RowBox[{"jnodes", ",", 
       RowBox[{"UndirectedEdge", "@@@", "jedges"}], ",", 
       RowBox[{"VertexCoordinates", "\[Rule]", "mcoords2"}], ",", 
       RowBox[{"VertexShapeFunction", "\[Rule]", "vsf"}]}], "]"}]}], 
    "\[IndentingNewLine]", ")"}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.5067964899577837`*^9, {3.506798489684682*^9, 3.506798521367976*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Main", "Section",
 CellChangeTimes->{{3.506796475539201*^9, 3.5067964791480103`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gg", "=", 
   RowBox[{"Graph", "[", 
    RowBox[{
     RowBox[{"Range", "@", "4"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "\[UndirectedEdge]", "2"}], ",", 
       RowBox[{"3", "\[UndirectedEdge]", "4"}]}], "}"}], ",", 
     RowBox[{"VertexCoordinates", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"1", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}]}], ",", 
        RowBox[{"2", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0", ",", "1"}], "}"}]}], ",", 
        RowBox[{"3", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}]}], ",", 
        RowBox[{"4", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1"}], "}"}]}]}], "}"}]}]}], "]"}]}], ";", 
  RowBox[{"Grid", "[", 
   RowBox[{
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Show", "[", 
        RowBox[{"twoColorGraph", "[", 
         RowBox[{"gg", ",", "#"}], "]"}], "]"}], "&"}], ",", 
      RowBox[{"Partition", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "1", "}"}], ",", 
          RowBox[{"{", "2", "}"}], ",", 
          RowBox[{"{", "3", "}"}], ",", 
          RowBox[{"{", "4", "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "3"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"2", ",", "4"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "4"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"2", ",", "3"}], "}"}]}], "}"}], ",", "3"}], "]"}], ",", 
      RowBox[{"{", "2", "}"}]}], "]"}], ",", 
    RowBox[{"Frame", "\[Rule]", "All"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5067964628897963`*^9, 3.506796493988721*^9}}],

Cell[BoxData[
 TagBox[GridBox[{
    {
     GraphicsBox[{GraphicsGroupBox[{
         {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 1.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{1., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 1.}, 0.25]}}, 
         {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
          LineBox[{{{5.551115123125783*^-17, 0.2764830508474574}, {
           5.551115123125783*^-17, 0.7235169491525424}}, {{1., 
           0.2764830508474574}, {1., 0.7235169491525424}}}]}}], {}},
      FrameTicks->None,
      ImageSize->60], 
     GraphicsBox[{GraphicsGroupBox[{
         {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{0., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 1.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{1., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 1.}, 0.25]}}, 
         {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
          LineBox[{{{5.551115123125783*^-17, 0.2764830508474574}, {
           5.551115123125783*^-17, 0.7235169491525424}}, {{1., 
           0.2764830508474574}, {1., 0.7235169491525424}}}]}}], {}},
      FrameTicks->None,
      ImageSize->60], 
     GraphicsBox[{GraphicsGroupBox[{
         {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 0.}, 0.25]}, 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{0., 1.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{1., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 1.}, 0.25]}}, 
         {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
          LineBox[{{{5.551115123125783*^-17, 0.2764830508474574}, {
           5.551115123125783*^-17, 0.7235169491525424}}, {{1., 
           0.2764830508474574}, {1., 0.7235169491525424}}}]}}], {}},
      FrameTicks->None,
      ImageSize->60]},
    {
     GraphicsBox[{GraphicsGroupBox[{
         {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 1.}, 0.25]}, 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 1.}, 0.25]}}, 
         {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
          LineBox[{{{5.551115123125783*^-17, 0.2764830508474574}, {
           5.551115123125783*^-17, 0.7235169491525424}}, {{1., 
           0.2764830508474574}, {1., 0.7235169491525424}}}]}}], {}},
      FrameTicks->None,
      ImageSize->60], 
     GraphicsBox[{GraphicsGroupBox[{
         {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 1.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{1., 0.}, 0.25]}, 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 1.}, 0.25]}}, 
         {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
          LineBox[{{{5.551115123125783*^-17, 0.2764830508474574}, {
           5.551115123125783*^-17, 0.7235169491525424}}, {{1., 
           0.2764830508474574}, {1., 0.7235169491525424}}}]}}], {}},
      FrameTicks->None,
      ImageSize->60], 
     GraphicsBox[{GraphicsGroupBox[{
         {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{0., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 1.}, 0.25]}, 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 1.}, 0.25]}}, 
         {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
          LineBox[{{{5.551115123125783*^-17, 0.2764830508474574}, {
           5.551115123125783*^-17, 0.7235169491525424}}, {{1., 
           0.2764830508474574}, {1., 0.7235169491525424}}}]}}], {}},
      FrameTicks->None,
      ImageSize->60]},
    {
     GraphicsBox[{GraphicsGroupBox[{
         {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 0.}, 0.25]}, 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{0., 1.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{1., 0.}, 0.25]}, 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 1.}, 0.25]}}, 
         {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
          LineBox[{{{5.551115123125783*^-17, 0.2764830508474574}, {
           5.551115123125783*^-17, 0.7235169491525424}}, {{1., 
           0.2764830508474574}, {1., 0.7235169491525424}}}]}}], {}},
      FrameTicks->None,
      ImageSize->60], 
     GraphicsBox[{GraphicsGroupBox[{
         {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{0., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 1.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{1., 0.}, 0.25]}, 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 1.}, 0.25]}}, 
         {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
          LineBox[{{{5.551115123125783*^-17, 0.2764830508474574}, {
           5.551115123125783*^-17, 0.7235169491525424}}, {{1., 
           0.2764830508474574}, {1., 0.7235169491525424}}}]}}], {}},
      FrameTicks->None,
      ImageSize->60], 
     GraphicsBox[{GraphicsGroupBox[{
         {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
          {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 0.}, 0.25]}, 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{0., 1.}, 0.25]}, 
          {RGBColor[1, 0.5, 0.5], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 0.}, 0.25]}, 
          {GrayLevel[1], EdgeForm[Thickness[Large]], 
           DiskBox[{1., 1.}, 0.25]}}, 
         {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
          LineBox[{{{5.551115123125783*^-17, 0.2764830508474574}, {
           5.551115123125783*^-17, 0.7235169491525424}}, {{1., 
           0.2764830508474574}, {1., 0.7235169491525424}}}]}}], {}},
      FrameTicks->None,
      ImageSize->60]}
   },
   AutoDelete->False,
   GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 CellChangeTimes->{{3.506796485810376*^9, 3.506796495904202*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"twoColorGraph", "[", 
    RowBox[{"gg", ",", 
     RowBox[{"{", "}"}]}], "]"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "200"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.506796500124115*^9, 3.506796518150646*^9}}],

Cell[BoxData[
 GraphicsBox[{GraphicsGroupBox[{
     {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
      {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 0.}, 0.25]}, 
      {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{0., 1.}, 0.25]}, 
      {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{1., 0.}, 0.25]}, 
      {GrayLevel[1], EdgeForm[Thickness[Large]], DiskBox[{1., 1.}, 0.25]}}, 
     {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
      LineBox[{{{5.551115123125783*^-17, 0.2764830508474574}, {
       5.551115123125783*^-17, 0.7235169491525424}}, {{1., 
       0.2764830508474574}, {1., 0.7235169491525424}}}]}}], {}},
  FrameTicks->None,
  ImageSize->200]], "Output",
 CellChangeTimes->{{3.506796500643982*^9, 3.506796519463964*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rowExpr", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Style", "[", 
      RowBox[{"\"\<(\>\"", ",", "30"}], "]"}], ",", 
     RowBox[{"myPlus", "@@", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"Show", "[", 
          RowBox[{
           RowBox[{"twoColorGraph", "[", 
            RowBox[{"gg1", ",", "#"}], "]"}], ",", 
           RowBox[{"ImageSize", "\[Rule]", "25"}]}], "]"}], "&"}], "/@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "1", "}"}], ",", 
          RowBox[{"{", "2", "}"}]}], "}"}]}], ")"}]}], ",", 
     RowBox[{"Style", "[", 
      RowBox[{"\"\<)\>\"", ",", "30"}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"expr", "=", 
  RowBox[{"Row", "[", 
   RowBox[{"rowExpr", "~", "Join", "~", 
    RowBox[{"{", 
     RowBox[{"Style", "[", 
      RowBox[{"\"\<\[Times]\>\"", ",", "30"}], "]"}], "}"}], "~", "Join", "~",
     "rowExpr"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.506796616261771*^9, 3.506796616450386*^9}}],

Cell[BoxData[
 TemplateBox[{
  StyleBox["\"(\"", StripOnInput -> False, FontSize -> 30],TemplateBox[{
     GraphicsBox[{
       GraphicsGroupBox[{{
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[{
            GrayLevel[0], 
            Opacity[0.7]}], {
           GrayLevel[1], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 0.}, 0.25]}, {
           GrayLevel[1], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 1.}, 0.25]}}, {
          Hue[0.6, 0.7, 0.5], 
          Opacity[0.7], 
          
          LineBox[{{-5.551115123125783*^-17, 
           0.258827683615819}, {-5.551115123125783*^-17, 
           0.7411723163841804}}]}}], {}}, FrameTicks -> None, ImageSize -> 
      25], 
     StyleBox["\"+\"", StripOnInput -> False, FontSize -> 30], 
     GraphicsBox[{
       GraphicsGroupBox[{{
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[{
            GrayLevel[0], 
            Opacity[0.7]}], {
           RGBColor[1, 0.5, 0.5], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 0.}, 0.25]}, {
           GrayLevel[1], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 1.}, 0.25]}}, {
          Hue[0.6, 0.7, 0.5], 
          Opacity[0.7], 
          
          LineBox[{{-5.551115123125783*^-17, 
           0.258827683615819}, {-5.551115123125783*^-17, 
           0.7411723163841804}}]}}], {}}, FrameTicks -> None, ImageSize -> 
      25], 
     StyleBox["\"+\"", StripOnInput -> False, FontSize -> 30], 
     GraphicsBox[{
       GraphicsGroupBox[{{
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[{
            GrayLevel[0], 
            Opacity[0.7]}], {
           GrayLevel[1], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 0.}, 0.25]}, {
           RGBColor[1, 0.5, 0.5], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 1.}, 0.25]}}, {
          Hue[0.6, 0.7, 0.5], 
          Opacity[0.7], 
          
          LineBox[{{-5.551115123125783*^-17, 
           0.258827683615819}, {-5.551115123125783*^-17, 
           0.7411723163841804}}]}}], {}}, FrameTicks -> None, ImageSize -> 
      25]}, "Row", 
    DisplayFunction -> (
     RowBox[{#, "\[InvisibleSpace]", #2, "\[InvisibleSpace]", #3, 
       "\[InvisibleSpace]", #4, "\[InvisibleSpace]", #5}]& ), 
    InterpretationFunction -> (RowBox[{"Row", "[", 
       RowBox[{"{", 
         RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5}], "}"}], "]"}]& )],
   StyleBox["\")\"", StripOnInput -> False, FontSize -> 30],StyleBox[
   "\"\[Times]\"", StripOnInput -> False, FontSize -> 30],StyleBox[
   "\"(\"", StripOnInput -> False, FontSize -> 30],TemplateBox[{
     GraphicsBox[{
       GraphicsGroupBox[{{
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[{
            GrayLevel[0], 
            Opacity[0.7]}], {
           GrayLevel[1], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 0.}, 0.25]}, {
           GrayLevel[1], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 1.}, 0.25]}}, {
          Hue[0.6, 0.7, 0.5], 
          Opacity[0.7], 
          
          LineBox[{{-5.551115123125783*^-17, 
           0.258827683615819}, {-5.551115123125783*^-17, 
           0.7411723163841804}}]}}], {}}, FrameTicks -> None, ImageSize -> 
      25], 
     StyleBox["\"+\"", StripOnInput -> False, FontSize -> 30], 
     GraphicsBox[{
       GraphicsGroupBox[{{
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[{
            GrayLevel[0], 
            Opacity[0.7]}], {
           RGBColor[1, 0.5, 0.5], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 0.}, 0.25]}, {
           GrayLevel[1], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 1.}, 0.25]}}, {
          Hue[0.6, 0.7, 0.5], 
          Opacity[0.7], 
          
          LineBox[{{-5.551115123125783*^-17, 
           0.258827683615819}, {-5.551115123125783*^-17, 
           0.7411723163841804}}]}}], {}}, FrameTicks -> None, ImageSize -> 
      25], 
     StyleBox["\"+\"", StripOnInput -> False, FontSize -> 30], 
     GraphicsBox[{
       GraphicsGroupBox[{{
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[{
            GrayLevel[0], 
            Opacity[0.7]}], {
           GrayLevel[1], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 0.}, 0.25]}, {
           RGBColor[1, 0.5, 0.5], 
           EdgeForm[
            Thickness[Large]], 
           DiskBox[{0., 1.}, 0.25]}}, {
          Hue[0.6, 0.7, 0.5], 
          Opacity[0.7], 
          
          LineBox[{{-5.551115123125783*^-17, 
           0.258827683615819}, {-5.551115123125783*^-17, 
           0.7411723163841804}}]}}], {}}, FrameTicks -> None, ImageSize -> 
      25]}, "Row", 
    DisplayFunction -> (
     RowBox[{#, "\[InvisibleSpace]", #2, "\[InvisibleSpace]", #3, 
       "\[InvisibleSpace]", #4, "\[InvisibleSpace]", #5}]& ), 
    InterpretationFunction -> (RowBox[{"Row", "[", 
       RowBox[{"{", 
         RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5}], "}"}], "]"}]& )],
   StyleBox["\")\"", StripOnInput -> False, FontSize -> 30]},
  "Row",
  DisplayFunction->(
   RowBox[{#, "\[InvisibleSpace]", #2, "\[InvisibleSpace]", #3, 
     "\[InvisibleSpace]", #4, "\[InvisibleSpace]", #5, 
     "\[InvisibleSpace]", #6, "\[InvisibleSpace]", #7}]& ),
  InterpretationFunction->(RowBox[{"Row", "[", 
     RowBox[{"{", 
       RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5, ",", #6, ",", #7}], 
       "}"}], "]"}]& )]], "Output",
 CellChangeTimes->{3.506796617333337*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"initGraph", "[", 
  RowBox[{"GraphData", "[", 
   RowBox[{"{", 
    RowBox[{"\"\<Cycle\>\"", ",", "4"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.506797170787936*^9, 3.5067971763555593`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"twoColorGraph", "[", 
  RowBox[{"g", ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.50679720457471*^9, 3.506797213492433*^9}}],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4}, {Null, {{1, 2}, {1, 4}, {2, 3}, {3, 4}}}, {
      ImageSize -> 60, 
       VertexCoordinates -> {{1., 2.}, {1., 1.}, {2., 1.}, {2., 2.}}, 
       VertexShapeFunction -> {$CellContext`frag$vsf}}]], Typeset`boxes = 
    GraphicsGroupBox[{{
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         GrayLevel[0], 
         EdgeForm[
          Thickness[Large]], 
         GrayLevel[1], 
         DiskBox[{1., 2.}, 
          NCache[
           Rational[1, 4], 0.25]]}, "DynamicName", BoxID -> "VertexID$1"], 
       TagBox[{
         GrayLevel[0], 
         EdgeForm[
          Thickness[Large]], 
         GrayLevel[1], 
         DiskBox[{1., 1.}, 
          NCache[
           Rational[1, 4], 0.25]]}, "DynamicName", BoxID -> "VertexID$2"], 
       TagBox[{
         GrayLevel[0], 
         EdgeForm[
          Thickness[Large]], 
         GrayLevel[1], 
         DiskBox[{2., 1.}, 
          NCache[
           Rational[1, 4], 0.25]]}, "DynamicName", BoxID -> "VertexID$3"], 
       TagBox[{
         GrayLevel[0], 
         EdgeForm[
          Thickness[Large]], 
         GrayLevel[1], 
         DiskBox[{2., 2.}, 
          NCache[
           Rational[1, 4], 0.25]]}, "DynamicName", BoxID -> "VertexID$4"]}, {
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}}]}}]}, 
    DynamicBox[GraphComputation`NetworkGraphicsBox[
     1, Typeset`graph, Typeset`boxes], {
     CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
      SynchronousUpdating -> False, TrackedSymbols :> {}},
     ImageSizeCache->{{2.3799999999999883`, 166.62}, {-84.75519999999997, 
      79.4848}}]]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImageSize->{170., Automatic}]], "Output",
 CellChangeTimes->{{3.506797204795373*^9, 3.5067972139481163`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"makeClusterGraph", "[", 
  RowBox[{"gg", ",", 
   RowBox[{"{", 
    RowBox[{"jnodes", ",", "jedges"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{{1, 2, 3}, {1, 3, 4}}, {Null, {{1, 2}}}, {
      VertexShapeFunction -> {$CellContext`vsf}, 
       VertexCoordinates -> {{1.2499992394605648`, 1.2499992394605648`}, {
        1.7500007605394352`, 1.7500007605394352`}}}]], Typeset`boxes = 
    GraphicsGroupBox[{{
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.9372549019607843, 0.8705882352941177, 0.803921568627451], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{2., 1.}, {1., 2.}, {1., 1.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["1", {1., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["2", {1., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 1.}, 0.25], 
                InsetBox["3", {2., 1.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{1., 1.7166666666666668`}, {1., 
                1.2833333333333332`}}, {{1.2833333333333332`, 1.}, {
                1.7166666666666668`, 1.}}}]}}], {}}}, {
          PlotRange -> {{0.5, 2.5}, {0.5, 2.5}}, ImageSize -> {60., 60.}}], {
         1.2499992394605648`, 1.2499992394605648`}], "DynamicName", BoxID -> 
        "VertexID$1"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.803921568627451, 0.5843137254901961, 0.4588235294117647], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{2., 1.}, {2., 2.}, {1., 2.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["1", {1., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 1.}, 0.25], 
                InsetBox["3", {2., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 2.}, 0.25], 
                InsetBox["4", {2., 2.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{1.2833333333333332`, 2.}, {1.7166666666666668`, 
                2.}}, {{2., 1.2833333333333332`}, {2., 
                1.7166666666666668`}}}]}}], {}}}, {
          PlotRange -> {{0.5, 2.5}, {0.5, 2.5}}, ImageSize -> {60., 60.}}], {
         1.7500007605394352`, 1.7500007605394352`}], "DynamicName", BoxID -> 
        "VertexID$2"]}, {
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{
         DynamicLocation["VertexID$1", Automatic, Center], 
         DynamicLocation["VertexID$2", Automatic, Center]}]}}]}, 
    DynamicBox[GraphComputation`NetworkGraphicsBox[
     1, Typeset`graph, Typeset`boxes], {
     CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
      SynchronousUpdating -> False, TrackedSymbols :> {}},
     ImageSizeCache->{{29.99999999999997, 
      192.00000000000003`}, {-84.00000000000006, 78.}}]]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImageSize->{223.00000000000006`, Automatic}]], "Output",
 CellChangeTimes->{3.506797077692318*^9, 3.50679711771917*^9, 
  3.506797187613514*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"makeClusterGraph", "[", 
  RowBox[{"g", ",", 
   RowBox[{"{", 
    RowBox[{"jnodes", ",", "jedges"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5067984004214087`*^9, 3.50679841188451*^9}}],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{{1, 2, 4}, {2, 3, 6}, {4, 7, 8}, {6, 8, 9}, {2, 4, 5, 6}, {4, 5, 
       6, 8}}, {Null, {{1, 5}, {2, 5}, {3, 6}, {4, 6}, {5, 6}}}, {
      VertexShapeFunction -> {$CellContext`vsf}, 
       VertexCoordinates -> {{0.3333331356494829, 0.3333330372144415}, {
        0.33333313564948297`, 1.6666669627855584`}, {1.6666668643505171`, 
        0.3333330372144415}, {1.6666668643505171`, 1.6666669627855586`}, {
        0.6464456838901884, 1.0000000000000002`}, {1.3535543161098114`, 
        0.9999999999999998}}}]], Typeset`boxes = GraphicsGroupBox[{{
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.9372549019607843, 0.8705882352941177, 0.803921568627451], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{1., 0.}, {0., 1.}, {0., 0.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{0., 0.}, 0.25], 
                InsetBox["1", {0., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{0., 1.}, 0.25], 
                InsetBox["2", {0., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 0.}, 0.25], 
                InsetBox["4", {1., 0.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{2.7755575615628914`*^-17, 0.2833333333333332}, {
                2.7755575615628914`*^-17, 0.7166666666666668}}, {{
                0.2833333333333333, -1.1102230246251565`*^-16}, {
                0.7166666666666668, -1.1102230246251565`*^-16}}}]}}], {}}}, {
          PlotRange -> {{-0.5, 1.5}, {-0.5, 1.5}}, 
           ImageSize -> {60., 60.}}], {0.3333331356494829, 
         0.3333330372144415}], "DynamicName", BoxID -> "VertexID$1"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.803921568627451, 0.5843137254901961, 0.4588235294117647], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{1., 2.}, {0., 2.}, {0., 1.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{0., 1.}, 0.25], 
                InsetBox["2", {0., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{0., 2.}, 0.25], 
                InsetBox["3", {0., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["6", {1., 2.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{2.7755575615628914`*^-17, 1.2833333333333332`}, {
                2.7755575615628914`*^-17, 1.7166666666666668`}}, {{
                0.2833333333333333, 2.}, {0.7166666666666668, 
                2.}}}]}}], {}}}, {
          PlotRange -> {{-0.5, 1.5}, {0.5, 2.5}}, ImageSize -> {60., 60.}}], {
         0.33333313564948297`, 1.6666669627855584`}], "DynamicName", BoxID -> 
        "VertexID$2"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.9921568627450981, 0.8509803921568627, 0.7098039215686275], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{2., 0.}, {2., 1.}, {1., 0.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{1., 0.}, 0.25], 
                InsetBox["4", {1., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 0.}, 0.25], 
                InsetBox["7", {2., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 1.}, 0.25], 
                InsetBox["8", {2., 1.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{1.2833333333333332`, -1.1102230246251565`*^-16}, {
                1.7166666666666668`, -1.1102230246251565`*^-16}}, {{2., 
                0.2833333333333332}, {2., 0.7166666666666668}}}]}}], {}}}, {
          PlotRange -> {{0.5, 2.5}, {-0.5, 1.5}}, ImageSize -> {60., 60.}}], {
         1.6666668643505171`, 0.3333330372144415}], "DynamicName", BoxID -> 
        "VertexID$3"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.47058823529411764`, 0.8588235294117647, 0.8862745098039215], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{2., 1.}, {2., 2.}, {1., 2.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["6", {1., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 1.}, 0.25], 
                InsetBox["8", {2., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 2.}, 0.25], 
                InsetBox["9", {2., 2.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{1.2833333333333332`, 2.}, {1.7166666666666668`, 
                2.}}, {{2., 1.2833333333333332`}, {2., 
                1.7166666666666668`}}}]}}], {}}}, {
          PlotRange -> {{0.5, 2.5}, {0.5, 2.5}}, ImageSize -> {60., 60.}}], {
         1.6666668643505171`, 1.6666669627855586`}], "DynamicName", BoxID -> 
        "VertexID$4"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.5294117647058824, 0.6627450980392157, 0.4196078431372549], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{1., 0.}, {1., 2.}, {0., 1.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{0., 1.}, 0.25], 
                InsetBox["2", {0., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 0.}, 0.25], 
                InsetBox["4", {1., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["5", {1., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["6", {1., 2.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{0.2833333333333333, 1.}, {0.7166666666666668, 
                1.}}, {{1., 0.2833333333333332}, {1., 0.7166666666666667}}, {{
                1., 1.2833333333333332`}, {1., 
                1.7166666666666668`}}}]}}], {}}}, {
          PlotRange -> {{-0.5, 1.5}, {-0.5, 2.5}}, 
           ImageSize -> {60., 90.}}], {0.6464456838901884, 
         1.0000000000000002`}], "DynamicName", BoxID -> "VertexID$5"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[1., 0.6431372549019608, 0.4549019607843137], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{2., 1.}, {1., 2.}, {1., 0.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{1., 0.}, 0.25], 
                InsetBox["4", {1., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["5", {1., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["6", {1., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 1.}, 0.25], 
                InsetBox["8", {2., 1.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{1., 0.2833333333333332}, {1., 
                0.7166666666666667}}, {{1., 1.2833333333333332`}, {1., 
                1.7166666666666668`}}, {{1.2833333333333332`, 1.}, {
                1.7166666666666668`, 1.}}}]}}], {}}}, {
          PlotRange -> {{0.5, 2.5}, {-0.5, 2.5}}, ImageSize -> {60., 90.}}], {
         1.3535543161098114`, 0.9999999999999998}], "DynamicName", BoxID -> 
        "VertexID$6"]}, {
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}}]}}]}, 
    DynamicBox[GraphComputation`NetworkGraphicsBox[
     1, Typeset`graph, Typeset`boxes], {
     CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
      SynchronousUpdating -> False, TrackedSymbols :> {}},
     ImageSizeCache->{{30.000000000000007`, 329.}, {-152.00000000000006`, 
      147.0000352889519}}]]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None]], "Output",
 CellChangeTimes->{{3.506798406220469*^9, 3.506798413140225*^9}, {
  3.506798610146674*^9, 3.5067986203606033`*^9}}]
}, Open  ]],

Cell[BoxData["g"], "Input",
 CellChangeTimes->{3.506798707658698*^9}],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9}, {Null, 
       SparseArray[
       Automatic, {9, 9}, 0, {
        1, {{0, 2, 5, 7, 10, 14, 17, 19, 22, 24}, {{4}, {2}, {1}, {3}, {5}, {
          2}, {6}, {1}, {5}, {7}, {2}, {4}, {6}, {8}, {3}, {5}, {9}, {4}, {
          8}, {5}, {7}, {9}, {6}, {8}}}, Pattern}]}, {
      Background -> Automatic, FrameTicks -> None, GraphStyle -> 
       "DiagramBlack", 
       VertexCoordinates -> {{0., 0.}, {0., 1.}, {0., 2.}, {1., 0.}, {1., 
        1.}, {1., 2.}, {2., 0.}, {2., 1.}, {2., 2.}}}]], Typeset`boxes = 
    GraphicsGroupBox[{{
       Directive[
        GrayLevel[0.7], 
        EdgeForm[]], 
       TagBox[{
         TagBox[
          StyleBox[
           
           RectangleBox[{-0.14973976041118808`, -0.09982678978350452}, {
            0.14973976041118808`, 0.09982678978350452}], 
           Directive[
            GrayLevel[0.7], 
            EdgeForm[]], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["1", TraditionalForm], 
          DynamicLocation["VertexID$1", None, Center], BaseStyle -> 
          "Graphics"]}, "DynamicName", BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          StyleBox[
           
           RectangleBox[{-0.14973976041118808`, 0.9001732102164954}, {
            0.14973976041118808`, 1.0998267897835046`}], 
           Directive[
            GrayLevel[0.7], 
            EdgeForm[]], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["2", TraditionalForm], 
          DynamicLocation["VertexID$2", None, Center], BaseStyle -> 
          "Graphics"]}, "DynamicName", BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          StyleBox[
           
           RectangleBox[{-0.14973976041118808`, 1.9001732102164954`}, {
            0.14973976041118808`, 2.0998267897835046`}], 
           Directive[
            GrayLevel[0.7], 
            EdgeForm[]], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          DynamicLocation["VertexID$3", None, Center], BaseStyle -> 
          "Graphics"]}, "DynamicName", BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          StyleBox[
           
           RectangleBox[{0.8502602395888119, -0.09982678978350452}, {
            1.149739760411188, 0.09982678978350452}], 
           Directive[
            GrayLevel[0.7], 
            EdgeForm[]], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["4", TraditionalForm], 
          DynamicLocation["VertexID$4", None, Center], BaseStyle -> 
          "Graphics"]}, "DynamicName", BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          StyleBox[
           
           RectangleBox[{0.8502602395888119, 0.9001732102164954}, {
            1.149739760411188, 1.0998267897835046`}], 
           Directive[
            GrayLevel[0.7], 
            EdgeForm[]], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["5", TraditionalForm], 
          DynamicLocation["VertexID$5", None, Center], BaseStyle -> 
          "Graphics"]}, "DynamicName", BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          StyleBox[
           
           RectangleBox[{0.8502602395888119, 1.9001732102164954`}, {
            1.149739760411188, 2.0998267897835046`}], 
           Directive[
            GrayLevel[0.7], 
            EdgeForm[]], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["6", TraditionalForm], 
          DynamicLocation["VertexID$6", None, Center], BaseStyle -> 
          "Graphics"]}, "DynamicName", BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          StyleBox[
           
           RectangleBox[{1.850260239588812, -0.09982678978350452}, {
            2.149739760411188, 0.09982678978350452}], 
           Directive[
            GrayLevel[0.7], 
            EdgeForm[]], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["7", TraditionalForm], 
          DynamicLocation["VertexID$7", None, Center], BaseStyle -> 
          "Graphics"]}, "DynamicName", BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          StyleBox[
           
           RectangleBox[{1.850260239588812, 0.9001732102164954}, {
            2.149739760411188, 1.0998267897835046`}], 
           Directive[
            GrayLevel[0.7], 
            EdgeForm[]], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["8", TraditionalForm], 
          DynamicLocation["VertexID$8", None, Center], BaseStyle -> 
          "Graphics"]}, "DynamicName", BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          StyleBox[
           
           RectangleBox[{1.850260239588812, 1.9001732102164954`}, {
            2.149739760411188, 2.0998267897835046`}], 
           Directive[
            GrayLevel[0.7], 
            EdgeForm[]], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["9", TraditionalForm], 
          DynamicLocation["VertexID$9", None, Center], BaseStyle -> 
          "Graphics"]}, "DynamicName", BoxID -> "VertexLabelID$9"]}, {
       Directive[
        GrayLevel[0], 
        Opacity[1]], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$8", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}], 
        Directive[
         GrayLevel[0], 
         Opacity[1]], StripOnInput -> False]}}]}, 
    DynamicBox[GraphComputation`NetworkGraphicsBox[
     1, Typeset`graph, Typeset`boxes], {
     CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
      SynchronousUpdating -> False, TrackedSymbols :> {}},
     ImageSizeCache->{{7.18, 351.82}, {-167.1072, 162.571147990181}}]]],
  Background->Automatic,
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None]], "Input",
 CellChangeTimes->{{3.50679871265508*^9, 3.50679871270507*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"initGraph", "[", 
   RowBox[{"GridGraph", "[", 
    RowBox[{"{", 
     RowBox[{"3", ",", "3"}], "}"}], "]"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initHardcore", "[", "g", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initMessages", "[", 
   RowBox[{"{", 
    RowBox[{"jnodes", ",", "jedges"}], "}"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"parallelUpdate", "[", "10", "]"}], "\[IndentingNewLine]"}], "Input",\

 CellChangeTimes->{{3.506799826329254*^9, 3.506799834345418*^9}, {
  3.506799878964521*^9, 3.506799879124542*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"jnodes", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "4", ",", "5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "3", ",", "5", ",", "6"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "5", ",", "7", ",", "8"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"5", ",", "6", ",", "8", ",", "9"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"edge1", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "4", ",", "5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "3", ",", "5", ",", "6"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"edge2", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "4", ",", "5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "5", ",", "7", ",", "8"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"edge3", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "3", ",", "5", ",", "6"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"5", ",", "6", ",", "8", ",", "9"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"edge4", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"4", ",", "5", ",", "7", ",", "8"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"5", ",", "6", ",", "8", ",", "9"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"jedges", "=", 
   RowBox[{"{", 
    RowBox[{"edge1", ",", "edge2", ",", "edge3", ",", "edge4"}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.506799990463086*^9, 3.506800088575509*^9}, {
  3.506800205102256*^9, 3.506800205532509*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"makeClusterGraph", "[", 
  RowBox[{"g", ",", 
   RowBox[{"{", 
    RowBox[{"jnodes", ",", "jedges"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5068001787090797`*^9, 3.50680018792586*^9}}],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{{1, 2, 4, 5}, {2, 3, 5, 6}, {4, 5, 7, 8}, {5, 6, 8, 9}}, {
      Null, {{1, 2}, {1, 3}, {2, 4}, {3, 4}}}, {
      VertexShapeFunction -> {$CellContext`vsf}, 
       VertexCoordinates -> {{0.500000018541405, 0.500000018541405}, {
        0.500000018541405, 1.499999981458595}, {1.499999981458595, 
        0.500000018541405}, {1.499999981458595, 1.499999981458595}}}]], 
    Typeset`boxes = GraphicsGroupBox[{{
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.9372549019607843, 0.8705882352941177, 0.803921568627451], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{1., 0.}, {1., 1.}, {0., 1.}, {0., 0.}}], CurveClosed -> 
             True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{0., 0.}, 0.25], 
                InsetBox["1", {0., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{0., 1.}, 0.25], 
                InsetBox["2", {0., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 0.}, 0.25], 
                InsetBox["4", {1., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["5", {1., 1.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{2.7755575615628914`*^-17, 0.2833333333333332}, {
                2.7755575615628914`*^-17, 0.7166666666666668}}, {{
                0.2833333333333333, -1.1102230246251565`*^-16}, {
                0.7166666666666668, -1.1102230246251565`*^-16}}, {{
                0.2833333333333333, 1.}, {0.7166666666666668, 1.}}, {{1., 
                0.2833333333333332}, {1., 0.7166666666666668}}}]}}], {}}}, {
          PlotRange -> {{-0.5, 1.5}, {-0.5, 1.5}}, 
           ImageSize -> {60., 60.}}], {0.500000018541405, 0.500000018541405}],
         "DynamicName", BoxID -> "VertexID$1"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.803921568627451, 0.5843137254901961, 0.4588235294117647], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{1., 1.}, {1., 2.}, {0., 2.}, {0., 1.}}], CurveClosed -> 
             True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{0., 1.}, 0.25], 
                InsetBox["2", {0., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{0., 2.}, 0.25], 
                InsetBox["3", {0., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["5", {1., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["6", {1., 2.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{2.7755575615628914`*^-17, 1.2833333333333332`}, {
                2.7755575615628914`*^-17, 1.7166666666666668`}}, {{
                0.2833333333333333, 1.}, {0.7166666666666668, 1.}}, {{
                0.2833333333333333, 2.}, {0.7166666666666668, 2.}}, {{1., 
                1.2833333333333332`}, {1., 1.7166666666666668`}}}]}}], {}}}, {
          PlotRange -> {{-0.5, 1.5}, {0.5, 2.5}}, ImageSize -> {60., 60.}}], {
         0.500000018541405, 1.499999981458595}], "DynamicName", BoxID -> 
        "VertexID$2"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.9921568627450981, 0.8509803921568627, 0.7098039215686275], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{2., 0.}, {2., 1.}, {1., 1.}, {1., 0.}}], CurveClosed -> 
             True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{1., 0.}, 0.25], 
                InsetBox["4", {1., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["5", {1., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 0.}, 0.25], 
                InsetBox["7", {2., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 1.}, 0.25], 
                InsetBox["8", {2., 1.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{1., 0.2833333333333332}, {1., 
                0.7166666666666668}}, {{
                1.2833333333333332`, -1.1102230246251565`*^-16}, {
                1.7166666666666668`, -1.1102230246251565`*^-16}}, {{
                1.2833333333333332`, 1.}, {1.7166666666666668`, 1.}}, {{2., 
                0.2833333333333332}, {2., 0.7166666666666668}}}]}}], {}}}, {
          PlotRange -> {{0.5, 2.5}, {-0.5, 1.5}}, ImageSize -> {60., 60.}}], {
         1.499999981458595, 0.500000018541405}], "DynamicName", BoxID -> 
        "VertexID$3"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.47058823529411764`, 0.8588235294117647, 0.8862745098039215], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{2., 1.}, {2., 2.}, {1., 2.}, {1., 1.}}], CurveClosed -> 
             True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["5", {1., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["6", {1., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 1.}, 0.25], 
                InsetBox["8", {2., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 2.}, 0.25], 
                InsetBox["9", {2., 2.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{1., 1.2833333333333332`}, {1., 
                1.7166666666666668`}}, {{1.2833333333333332`, 1.}, {
                1.7166666666666668`, 1.}}, {{1.2833333333333332`, 2.}, {
                1.7166666666666668`, 2.}}, {{2., 1.2833333333333332`}, {2., 
                1.7166666666666668`}}}]}}], {}}}, {
          PlotRange -> {{0.5, 2.5}, {0.5, 2.5}}, ImageSize -> {60., 60.}}], {
         1.499999981458595, 1.499999981458595}], "DynamicName", BoxID -> 
        "VertexID$4"]}, {
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}}]}}]}, 
    DynamicBox[GraphComputation`NetworkGraphicsBox[
     1, Typeset`graph, Typeset`boxes], {
     CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
      SynchronousUpdating -> False, TrackedSymbols :> {}},
     ImageSizeCache->{{30., 247.}, {-111., 106.}}]]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImageSize->{278., Automatic}]], "Output",
 CellChangeTimes->{3.506800189180204*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"initMessages", "[", 
   RowBox[{"{", 
    RowBox[{"jnodes", ",", "jedges"}], "}"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"vals2", "=", 
    RowBox[{"{", "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Clear", "[", "doOne", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"doOne", "[", "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"val", "=", 
      RowBox[{"Exp", "[", 
       RowBox[{"betheFreeEnergy", "[", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"vals2", ",", "val"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"parallelUpdate", "[", "1", "]"}], ";"}], "\[IndentingNewLine]", 
    ")"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.506800482567245*^9, 3.506800493976389*^9}, {
  3.506800530373691*^9, 3.506800530499998*^9}, {3.506800570736663*^9, 
  3.5068005710973577`*^9}, {3.5068006956579027`*^9, 3.506800697199135*^9}, {
  3.5068007325815983`*^9, 3.506800735757077*^9}, {3.5068008208474693`*^9, 
  3.506800821120554*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"doOne", "[", "]"}], "&"}], "/@", 
  RowBox[{"Range", "[", "20", "]"}]}]], "Input",
 CellChangeTimes->{{3.506800789787388*^9, 3.5068007898584843`*^9}, {
  3.506800826758762*^9, 3.506800827309908*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "Null", ",", "Null", ",", "Null", ",", "Null", ",", "Null", ",", "Null", 
   ",", "Null", ",", "Null", ",", "Null", ",", "Null", ",", "Null", ",", 
   "Null", ",", "Null", ",", "Null", ",", "Null", ",", "Null", ",", "Null", 
   ",", "Null", ",", "Null", ",", "Null"}], "}"}]], "Output",
 CellChangeTimes->{3.506800795438263*^9, 3.506800830095683*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"vals3", "=", 
   RowBox[{"Thread", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Range", "[", "20", "]"}], ",", "vals2"}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"vals3", ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "21"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "50"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"PointSize", "\[Rule]", "Large"}]}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<iteration\>\"", ",", "\"\<#\>\""}], "}"}]}], ",", 
   RowBox[{"Filling", "\[Rule]", "Axis"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.50680079712324*^9, 3.506800811695957*^9}, {
  3.506800842406663*^9, 3.506800912186893*^9}, {3.506800966489356*^9, 
  3.50680102031959*^9}}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwAWIQDQEf7BlBVIOhA1TA4dDLpSVRax2gfA6HD193+39Y
AeMLOLy81X6177EjlC/iYD9t3glDIWcoX8KBc4mIK9sTGF/GIesfy+cSI1co
X8Fh0YFdQvMWw/hKDv9WsDKtKXaD8lUcbPNWzC9eDuOrObxc0a99ltcdytdw
8NyT+rJSGcbXcqhl4DlbFgDj6zjUFi/vl42E8fUchJJX8+7NhvENHLgemSlJ
5sP4hg5Rfo6VnWUwvpFD9P/PCy+Ww/jGDh9czKbvqILxTRzi2G5bm1TD+B/s
GVCBAyqXA40vgMYXQeNLoPFl0PgKaHwlNL4KGl8Nja+BxtdC4+ug8fXQ+AZo
fEM0vhEa3xiNb4LGH01/5KY/ANpYog8=
   "], {{{}, {}, {}, {}, 
     {Hue[0.67, 0.6, 0.6], Opacity[0.2], LineBox[{21, 1}], LineBox[{22, 2}], 
      LineBox[{23, 3}], LineBox[{24, 4}], LineBox[{25, 5}], LineBox[{26, 6}], 
      LineBox[{27, 7}], LineBox[{28, 8}], LineBox[{29, 9}], LineBox[{30, 10}],
       LineBox[{31, 11}], LineBox[{32, 12}], LineBox[{33, 13}], 
      LineBox[{34, 14}], LineBox[{35, 15}], LineBox[{36, 16}], 
      LineBox[{37, 17}], LineBox[{38, 18}], LineBox[{39, 19}], 
      LineBox[{40, 20}]}}, {{}, 
     {Hue[0.67, 0.6, 0.6], PointSize[Large], 
      PointBox[{41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 
       56, 57, 58, 59, 60}]}, {}}}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["\"iteration\"", TraditionalForm], 
    FormBox["\"#\"", TraditionalForm]},
  AxesOrigin->{0, 0},
  PlotRange->{{0, 21}, {0, 50}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.506800799325725*^9, 3.506800912433404*^9}, {
  3.506800968065899*^9, 3.5068010206490707`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Exp", "[", 
  RowBox[{"betheFreeEnergy", "[", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.506801100521214*^9, 3.506801105346478*^9}}],

Cell[BoxData["46.97122885363111`"], "Output",
 CellChangeTimes->{3.506801105809478*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{740, 752},
WindowMargins->{{80, Automatic}, {11, Automatic}},
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 6, \
2010)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 91, 1, 67, "Section"],
Cell[673, 25, 17015, 426, 1708, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17725, 456, 91, 1, 67, "Section"],
Cell[CellGroupData[{
Cell[17841, 461, 1808, 52, 88, "Input"],
Cell[19652, 515, 7124, 136, 217, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[26813, 656, 274, 7, 27, "Input"],
Cell[27090, 665, 764, 13, 215, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[27891, 683, 1077, 31, 88, "Input"],
Cell[28971, 716, 5619, 158, 86, "Output"]
}, Open  ]],
Cell[34605, 877, 227, 5, 27, "Input"],
Cell[CellGroupData[{
Cell[34857, 886, 173, 4, 27, "Input"],
Cell[35033, 892, 2585, 69, 185, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37655, 966, 152, 4, 27, "Input"],
Cell[37810, 972, 3900, 96, 238, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41747, 1073, 218, 5, 27, "Input"],
Cell[41968, 1080, 10391, 243, 375, "Output"]
}, Open  ]],
Cell[52374, 1326, 69, 1, 27, "Input"],
Cell[52446, 1329, 9177, 240, 359, InheritFromParent],
Cell[61626, 1571, 612, 17, 88, "Input"],
Cell[62241, 1590, 1794, 56, 103, "Input"],
Cell[CellGroupData[{
Cell[64060, 1650, 218, 5, 27, "Input"],
Cell[64281, 1657, 8237, 192, 293, "Output"]
}, Open  ]],
Cell[72533, 1852, 1159, 30, 148, "Input"],
Cell[CellGroupData[{
Cell[73717, 1886, 246, 6, 27, "Input"],
Cell[73966, 1894, 392, 7, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[74395, 1906, 917, 25, 58, "Input"],
Cell[75315, 1933, 1544, 30, 236, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[76896, 1968, 159, 3, 27, "Input"],
Cell[77058, 1973, 87, 1, 27, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
