(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     47960,       1292]
NotebookOptionsPosition[     46922,       1252]
NotebookOutlinePosition[     47279,       1268]
CellTagsIndexPosition[     47236,       1265]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Bulatov`common`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Bulatov`treeDecomposition`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"$Post", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"$ContextPath", "=", 
       RowBox[{"DeleteCases", "[", 
        RowBox[{"$ContextPath", ",", 
         RowBox[{"\"\<Combinatorica`\>\"", "|", "\"\<GraphUtilities`\>\""}]}],
         "]"}]}], ";", "#"}], ")"}], "&"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"On", "[", "Assert", "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.506535697822318*^9, 3.50653574267903*^9}, {
  3.506738133649156*^9, 3.506738133855921*^9}, {3.506739236013761*^9, 
  3.506739237920066*^9}}],

Cell[CellGroupData[{

Cell["Utils", "Section",
 CellChangeTimes->{{3.506540598216406*^9, 3.506540600703907*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"getLabelCoordinateMap", "[", "gp1_", "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "points", ",", "labels", ",", "actualLabels", ",", "coordAliases", ",",
         "actualCoords"}], "}"}], ",", 
      RowBox[{
       RowBox[{"points", "=", 
        RowBox[{
         RowBox[{"Cases", "[", 
          RowBox[{"gp1", ",", 
           RowBox[{
            RowBox[{"GraphicsComplex", "[", 
             RowBox[{"points_", ",", "__"}], "]"}], "\[RuleDelayed]", 
            "points"}], ",", "Infinity"}], "]"}], "//", "First"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"labels", "=", 
        RowBox[{"Cases", "[", 
         RowBox[{"gp1", ",", 
          RowBox[{"Text", "[", "___", "]"}], ",", "Infinity"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"actualLabels", "=", 
        RowBox[{"labels", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "1", ",", "1"}], "]"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"coordAliases", "=", 
        RowBox[{"labels", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "2"}], "]"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"actualCoords", "=", 
        RowBox[{"points", "[", 
         RowBox[{"[", "coordAliases", "]"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Thread", "[", 
        RowBox[{"actualLabels", "\[Rule]", "actualCoords"}], "]"}]}]}], 
     "]"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "gets", " ", "all", " ", "pairs", " ", "of", " ", "sets", " ", "that", " ",
     "intersect"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"intersectingPairs", "[", "bags_List", "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"cands", ",", "result"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"cands", "=", 
        RowBox[{
         RowBox[{"Outer", "[", 
          RowBox[{"List", ",", "bags", ",", "bags", ",", "1"}], "]"}], "//", 
         "fl1"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Select", "[", 
        RowBox[{"cands", ",", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
            "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
            "\[NotEqual]", 
            RowBox[{
            "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], "&&", 
           RowBox[{"!", 
            RowBox[{"disjoint", "[", 
             RowBox[{
              RowBox[{
              "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
              ",", 
              RowBox[{
              "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
             "]"}]}], "&&", 
           RowBox[{"OrderedQ", "[", "#", "]"}]}], "&"}]}], "]"}]}]}], 
     "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"independentClusterSet", "[", "jnodes_", "]"}], ":=", 
    RowBox[{"FindIndependentVertexSet", "@", 
     RowBox[{"Graph", "[", 
      RowBox[{"jnodes", ",", 
       RowBox[{"UndirectedEdge", "@@@", 
        RowBox[{"intersectingPairs", "[", "jnodes", "]"}]}]}], "]"}]}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"correctPR", "[", 
    RowBox[{"pr_", ",", "scale_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"pr", "[", 
       RowBox[{"[", "1", "]"}], "]"}], "+", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "1"}], "/", "scale"}], ",", 
        RowBox[{"2", "/", "scale"}]}], "}"}]}], ",", 
     RowBox[{
      RowBox[{"pr", "[", 
       RowBox[{"[", "2", "]"}], "]"}], "+", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "2"}], "/", "scale"}], ",", 
        RowBox[{"1", "/", "scale"}]}], "}"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"getPlotRange", "[", 
    RowBox[{"lst_", ",", 
     RowBox[{"offset_:", 
      RowBox[{"1", "/", "2"}]}]}], "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"xs", ",", "ys"}], "}"}], "=", 
      RowBox[{"Transpose", "[", "lst", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{"two", " ", "extra", " ", 
       RowBox[{"1", "/", "scale"}], " ", "offsets", " ", "needed", " ", "for",
        " ", "exact", " ", "match"}], "*)"}], 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"Min", "[", "xs", "]"}], "-", "offset"}], ",", 
         RowBox[{
          RowBox[{"Max", "[", "xs", "]"}], "+", "offset"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"Min", "[", "ys", "]"}], "-", "offset"}], ",", 
         RowBox[{
          RowBox[{"Max", "[", "ys", "]"}], "+", "offset"}]}], "}"}]}], 
      "}"}]}], "\[IndentingNewLine]", ")"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.506540643013784*^9, 3.506540643303092*^9}, {
  3.506738396310212*^9, 3.5067383965963306`*^9}, {3.5067396489481077`*^9, 
  3.506739649224311*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Main", "Section",
 CellChangeTimes->{{3.5065406039616528`*^9, 3.506540606591539*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"checkClusterDecomposition", "[", 
     RowBox[{"g_", ",", 
      RowBox[{"{", 
       RowBox[{"jnodes_", ",", "jedges_"}], "}"}]}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"isok", "=", "True"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"existNodes", "=", 
       RowBox[{"Union", "[", 
        RowBox[{"VertexList", "[", "g", "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"coveredNodes", "=", 
       RowBox[{"Union", "[", 
        RowBox[{"Flatten", "[", "jnodes", "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"prop", "=", 
       RowBox[{"(", 
        RowBox[{"existNodes", "===", "coveredNodes"}], ")"}]}], ";", "\n", 
      "\t", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", "prop"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<checkClusterDecomposition: violates variable preservation for \
\>\"", ",", 
           RowBox[{"existNodes", "\[Backslash]", "coveredNodes"}]}], "]"}], 
         ";", 
         RowBox[{"isok", "=", "False"}]}]}], "]"}], ";", "\n", 
      "\[IndentingNewLine]", 
      RowBox[{"existFactors", "=", 
       RowBox[{"deepSort", "@", 
        RowBox[{"Union", "[", 
         RowBox[{"List", "@@@", 
          RowBox[{"EdgeList", "[", "g", "]"}]}], "]"}]}]}], ";", 
      RowBox[{"coveredFactors", "=", 
       RowBox[{"Union", "[", 
        RowBox[{"Select", "[", 
         RowBox[{"existFactors", ",", 
          RowBox[{
           RowBox[{"anyTrue", "[", 
            RowBox[{
             RowBox[{"bag", "\[Element]", "jnodes"}], ",", 
             RowBox[{"#", "\[Subset]", "bag"}]}], "]"}], "&"}]}], "]"}], 
        "]"}]}], ";", 
      RowBox[{"prop", "=", 
       RowBox[{"(", 
        RowBox[{"existFactors", "===", "coveredFactors"}], ")"}]}], ";", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", "prop"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<checkClusterDecomposition: violates factor preservation for \>\
\"", ",", 
           RowBox[{"existFactors", "\[Backslash]", "coveredFactors"}]}], 
          "]"}], ";", 
         RowBox[{"isok", "=", "False"}]}]}], "]"}], ";", "\n", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
       "*", " ", "maybe", " ", "check", " ", "separator", " ", "properties", 
        " ", "too", " ", "later"}], " ", "**)"}], "\[IndentingNewLine]", 
      "isok"}], "\[IndentingNewLine]", ")"}]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.50648428477975*^9, 3.506484285558354*^9}, {
  3.506738330715225*^9, 3.506738355459305*^9}, {3.50673844570669*^9, 
  3.506738447075595*^9}, {3.506739636573394*^9, 3.506739646347262*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"initGraph", "[", "gg_Graph", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"g", "=", "gg"}], ";", "\[IndentingNewLine]", 
      RowBox[{"lineThickness", "=", 
       RowBox[{"1", "/", "2"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"masterScale", "=", "100"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"jnodes", ",", "jedges"}], "}"}], "=", 
       RowBox[{"findTreeDecomposition", "[", 
        RowBox[{"g", ",", 
         RowBox[{"PrimaryHeuristic", "\[Rule]", "\"\<minfill\>\""}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"gap", "=", "lineThickness"}], ";", "\[IndentingNewLine]", 
      RowBox[{"colorList", "=", 
       RowBox[{"ColorData", "[", 
        RowBox[{"\"\<Crayola\>\"", ",", "\"\<ColorList\>\""}], "]"}]}], ";"}],
      "\[IndentingNewLine]", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"drawFragment", "[", 
     RowBox[{"nodes_", ",", "scale_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"OrderedQ", "[", "nodes", "]"}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"points", "=", 
       RowBox[{"nodes", "/.", "vcoords"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"plotRange", "=", 
       RowBox[{"getPlotRange", "[", "points", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"imageSize", "=", 
       RowBox[{
        RowBox[{"-", "scale"}], "*", 
        RowBox[{"Subtract", "@@@", "plotRange"}]}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"Clear", "[", "frag$vsf", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"frag$vsf", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"x_", ",", "y_"}], "}"}], ",", "v_", ",", 
         RowBox[{"{", 
          RowBox[{"w_", ",", "h_"}], "}"}]}], "]"}], ":=", 
       RowBox[{"{", 
        RowBox[{"Black", ",", 
         RowBox[{"Circle", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"x", ",", "y"}], "}"}], ",", 
           RowBox[{"lineThickness", "/", "2"}]}], "]"}], ",", 
         RowBox[{"Text", "[", 
          RowBox[{"v", ",", 
           RowBox[{"{", 
            RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], "}"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"hull", "=", 
       RowBox[{"ComputationalGeometry`ConvexHull", "[", 
        RowBox[{"points", ",", 
         RowBox[{"AllPoints", "\[Rule]", "False"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"coverP", "=", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"colorList", "\[LeftDoubleBracket]", 
            RowBox[{"idx", "[", 
             RowBox[{"jnodes", ",", "nodes"}], "]"}], 
            "\[RightDoubleBracket]"}], ",", 
           RowBox[{"AbsoluteThickness", "[", 
            RowBox[{"scale", "*", "lineThickness"}], "]"}], ",", 
           RowBox[{"JoinForm", "[", "\"\<Round\>\"", "]"}], ",", 
           RowBox[{"JoinedCurve", "[", 
            RowBox[{
             RowBox[{"Line", "[", 
              RowBox[{"points", "[", 
               RowBox[{"[", "hull", "]"}], "]"}], "]"}], ",", 
             RowBox[{"CurveClosed", "\[Rule]", "True"}]}], "]"}]}], "}"}], 
         ",", 
         RowBox[{"PlotRange", "\[Rule]", "plotRange"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "imageSize"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"graphP", "=", 
       RowBox[{"Show", "[", 
        RowBox[{"Subgraph", "[", 
         RowBox[{"g", ",", "nodes", ",", 
          RowBox[{"VertexCoordinates", "\[Rule]", "vcoords"}], ",", 
          RowBox[{"VertexShapeFunction", "\[Rule]", "frag$vsf"}], ",", 
          RowBox[{"PlotRange", "\[Rule]", "plotRange"}], ",", 
          RowBox[{"ImageSize", "\[Rule]", "imageSize"}]}], "]"}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Show", "[", 
       RowBox[{"coverP", ",", "graphP"}], "]"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"makeClusterGraph", "[", 
    RowBox[{"g_Graph", ",", 
     RowBox[{"{", 
      RowBox[{"jnodes_", ",", "jedges_"}], "}"}], ",", 
     RowBox[{"gap_:", 
      RowBox[{"1", "/", "2"}]}]}], "]"}], ":=", 
   RowBox[{"(", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Assert", "[", 
      RowBox[{"checkClusterDecomposition", "[", 
       RowBox[{"g", ",", 
        RowBox[{"{", 
         RowBox[{"jnodes", ",", "jedges"}], "}"}]}], "]"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"method", "=", "\"\<minimize\>\""}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"vcoords", "=", 
      RowBox[{"Thread", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"VertexList", "[", "g", "]"}], "->", "VertexCoordinates"}], "/.", 
        RowBox[{"AbsoluteOptions", "[", "g", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"meanBagCoord", "[", "bag_", "]"}], ":=", 
      RowBox[{"Mean", "[", 
       RowBox[{"bag", "/.", "vcoords"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"mcoords", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "\[Rule]", 
         RowBox[{"meanBagCoord", "[", "#", "]"}]}], "&"}], "/@", "jnodes"}]}],
      ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"Switch", "[", 
      RowBox[{
      "method", ",", "\[IndentingNewLine]", "\"\<independentSet\>\"", ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"fixedClusters", "=", 
         RowBox[{"independentClusterSet", "[", "jnodes", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"fcoords", "=", 
         RowBox[{"Select", "[", 
          RowBox[{"mcoords", ",", 
           RowBox[{
            RowBox[{"MemberQ", "[", 
             RowBox[{"fixedClusters", ",", 
              RowBox[{"First", "[", "#", "]"}]}], "]"}], "&"}]}], "]"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"tempPlot", "=", 
         RowBox[{"GraphPlot", "[", 
          RowBox[{
           RowBox[{"Rule", "@@@", "jedges"}], ",", 
           RowBox[{"VertexCoordinateRules", "\[Rule]", "fcoords"}], ",", 
           RowBox[{"VertexLabeling", "\[Rule]", "True"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"mcoords2", "=", 
         RowBox[{"getLabelCoordinateMap", "[", "tempPlot", "]"}]}]}], ",", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", "\"\<minimize\>\"", ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "distance", " ", "of", " ", "cluster", " ", "from", " ", "its", " ", 
         "average", " ", "position"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"objCl", "[", "cluster_", "]"}], ":=", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"k", "=", 
            RowBox[{"idx", "[", 
             RowBox[{"jnodes", ",", "cluster"}], "]"}]}], ";", 
           RowBox[{"Total", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"cluster", "/.", "mcoords"}], ")"}], "-", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"xcl", "[", "k", "]"}], ",", 
                 RowBox[{"ycl", "[", "k", "]"}]}], "}"}]}], ")"}], "^", "2"}],
             "]"}]}], ")"}]}], ";", "\[IndentingNewLine]", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "constrain", " ", "two", " ", "cluster", " ", "to", " ", "be", " ", 
          "at", " ", "a", " ", "distance", " ", "from", " ", "each", " ", 
          "other"}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"consCl", "[", 
          RowBox[{"{", 
           RowBox[{"bag1_", ",", "bag2_"}], "}"}], "]"}], ":=", 
         RowBox[{"(", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"k1", "=", 
            RowBox[{"idx", "[", 
             RowBox[{"jnodes", ",", "bag1"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"k2", "=", 
            RowBox[{"idx", "[", 
             RowBox[{"jnodes", ",", "bag2"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"xcl", "[", "k1", "]"}], "-", 
                RowBox[{"xcl", "[", "k2", "]"}]}], ")"}], "^", "2"}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"ycl", "[", "k1", "]"}], "-", 
                RowBox[{"ycl", "[", "k2", "]"}]}], ")"}], "^", "2"}]}], 
            "\[GreaterEqual]", "gap"}]}], "\[IndentingNewLine]", ")"}]}], ";",
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"obj", "=", 
         RowBox[{"Total", "[", 
          RowBox[{"objCl", "/@", "jnodes"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"cons", "=", 
           RowBox[{"consCl", "/@", "jedges"}]}], ";"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"cons", "=", 
         RowBox[{"consCl", "/@", 
          RowBox[{"Select", "[", 
           RowBox[{
            RowBox[{"fl1", "@", 
             RowBox[{"Outer", "[", 
              RowBox[{"List", ",", "jnodes", ",", "jnodes", ",", "1"}], 
              "]"}]}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"First", "@", "#"}], "\[NotEqual]", 
              RowBox[{"Last", "@", "#"}]}], "&"}]}], "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Clear", "[", 
         RowBox[{"xcl", ",", "ycl"}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"varlist", "=", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"xcl", "[", "#", "]"}], ",", 
             RowBox[{"ycl", "[", "#", "]"}]}], "}"}], "&"}], "/@", 
          RowBox[{"Range", "[", 
           RowBox[{"Length", "@", "jnodes"}], "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"vars", "=", 
         RowBox[{"Flatten", "@", "varlist"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"sol", "=", 
         RowBox[{"Minimize", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"obj", ",", "cons"}], "}"}], ",", "vars"}], "]"}]}], ";", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"tempC", "=", 
         RowBox[{"varlist", "/.", 
          RowBox[{"Last", "@", 
           RowBox[{"FindMinimum", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"obj", ",", "cons"}], "}"}], ",", "vars"}], "]"}]}]}]}],
         ";", "\[IndentingNewLine]", 
        RowBox[{"mcoords2", "=", 
         RowBox[{"Thread", "[", 
          RowBox[{"jnodes", "\[Rule]", "tempC"}], "]"}]}]}], ",", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", "True", ",", 
       RowBox[{"Assert", "[", "\"\<Unknown arrangement method\>\"", "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"Clear", "[", "vsf", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"vsf", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x_", ",", "y_"}], "}"}], ",", "v_", ",", 
        RowBox[{"{", 
         RowBox[{"w_", ",", "h_"}], "}"}]}], "]"}], ":=", 
      RowBox[{"Inset", "[", 
       RowBox[{
        RowBox[{"drawFragment", "[", 
         RowBox[{"v", ",", "30"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Graph", "[", 
      RowBox[{"jnodes", ",", 
       RowBox[{"UndirectedEdge", "@@@", "jedges"}], ",", 
       RowBox[{"VertexCoordinates", "\[Rule]", "mcoords2"}], ",", 
       RowBox[{"VertexShapeFunction", "\[Rule]", "vsf"}]}], "]"}]}], 
    "\[IndentingNewLine]", ")"}]}], ";"}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBmIQ7X1akI1T4ZXjqpD9YHrq7I9sekD69vRfYNpFnNND
H0j/SrUE0+fy360C0auC8teAaIElhbcNgXTQvzYwfVyAv3kSkH4j59QOoq9o
Pz4Iok89+nIGRE+axn8VRHu0vb8Ool/yrH4CotMMyl+D6LLX8z+C6BflVT9A
tKnUTObJQLpFKJ8PRO/Y/VoERF81/CEKol2+ZsiCaCVVAX0QveaIrTmINnvq
D6Zf3E13AdFqsws9QLTjvBtBINoi9D2YTpqwj3s1kOZp0eAF0XatUVIgOoVp
qgyIfrBeQxFER05bD6YBhTWNVg==
  "]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"initGraph", "[", 
  RowBox[{"GridGraph", "[", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3"}], "}"}], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"makeClusterGraph", "[", 
  RowBox[{"g", ",", 
   RowBox[{"{", 
    RowBox[{"jnodes", ",", "jedges"}], "}"}], ",", "1"}], "]"}]}], "Input",
 CellChangeTimes->{
  3.506741061594522*^9, {3.506788405217164*^9, 3.5067884307173*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMinimize", "::", "incst"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(NMinimize\\)\[NoBreak] was unable to \
generate any initial points satisfying the inequality constraints \
\[NoBreak]\\!\\({\\(\\(\\(\\(1 - \\((\\(\\(xcl[1]\\)\\) - \\(\\(xcl[2]\\)\\))\
\\)\\^2 - \\((\\(\\(ycl[1]\\)\\) - \\(\\(ycl[2]\\)\\))\\)\\^2\\)\\) \
\[LessEqual] 0\\)\\), \\(\\(\\(\\(1 - \\((\\(\\(-\\(\\(xcl[1]\\)\\)\\)\\) + \
\\(\\(xcl[2]\\)\\))\\)\\^2 - \\((\\(\\(-\\(\\(ycl[1]\\)\\)\\)\\) + \
\\(\\(ycl[2]\\)\\))\\)\\^2\\)\\) \[LessEqual] 0\\)\\), \\(\\(\\(\\(1 - \
\\((\\(\\(xcl[1]\\)\\) - \\(\\(xcl[3]\\)\\))\\)\\^2 - \\((\\(\\(ycl[1]\\)\\) \
- \\(\\(ycl[3]\\)\\))\\)\\^2\\)\\) \[LessEqual] 0\\)\\), \\(\\(\\(\\(1 - \\((\
\\(\\(xcl[2]\\)\\) - \\(\\(xcl[3]\\)\\))\\)\\^2 - \\((\\(\\(ycl[2]\\)\\) - \
\\(\\(ycl[3]\\)\\))\\)\\^2\\)\\) \[LessEqual] 0\\)\\), \\(\\(\[LeftSkeleton] \
23 \[RightSkeleton]\\)\\), \\(\\(\\(\\(1 - \
\\((\\(\\(-\\(\\(xcl[3]\\)\\)\\)\\) + \\(\\(xcl[6]\\)\\))\\)\\^2 - \
\\((\\(\\(-\\(\\(ycl[3]\\)\\)\\)\\) + \\(\\(ycl[6]\\)\\))\\)\\^2\\)\\) \
\[LessEqual] 0\\)\\), \\(\\(\\(\\(1 - \\((\\(\\(-\\(\\(xcl[4]\\)\\)\\)\\) + \
\\(\\(xcl[6]\\)\\))\\)\\^2 - \\((\\(\\(-\\(\\(ycl[4]\\)\\)\\)\\) + \
\\(\\(ycl[6]\\)\\))\\)\\^2\\)\\) \[LessEqual] 0\\)\\), \\(\\(\\(\\(1 - \
\\((\\(\\(-\\(\\(xcl[5]\\)\\)\\)\\) + \\(\\(xcl[6]\\)\\))\\)\\^2 - \
\\((\\(\\(-\\(\\(ycl[5]\\)\\)\\)\\) + \\(\\(ycl[6]\\)\\))\\)\\^2\\)\\) \
\[LessEqual] 0\\)\\)}\\)\[NoBreak]. The initial region specified may not \
contain any feasible points. Changing the initial region or specifying \
explicit initial points may provide a better solution. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/NMinimize/incst\\\", \
ButtonNote -> \\\"NMinimize::incst\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.506788589887246*^9}],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{{6, 8, 9}, {1, 2, 3, 4}, {2, 3, 4, 5}, {3, 4, 5, 6}, {4, 5, 6, 
       7}, {5, 6, 7, 8}}, {Null, {{1, 6}, {2, 3}, {3, 4}, {4, 5}, {5, 6}}}, {
      VertexShapeFunction -> {$CellContext`vsf}, 
       VertexCoordinates -> {{1.7339786718948398`, 1.9736705181885228`}, {
        0.1588685363712495, 0.4025341331653798}, {-0.024052427183611343`, 
        1.3856617594028307`}, {0.9740256436370539, 1.3236924959875633`}, {
        1.1569466128675054`, 0.340564871583384}, {1.916899629078195, 
        0.9905428883461539}}}]], Typeset`boxes = GraphicsGroupBox[{{
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.9372549019607843, 0.8705882352941177, 0.803921568627451], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{2., 1.}, {2., 2.}, {1., 2.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["6", {1., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 1.}, 0.25], 
                InsetBox["8", {2., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 2.}, 0.25], 
                InsetBox["9", {2., 2.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{1.2833333333333332`, 2.}, {1.7166666666666668`, 
                2.}}, {{2., 1.2833333333333332`}, {2., 
                1.7166666666666668`}}}]}}], {}}}, {
          PlotRange -> {{0.5, 2.5}, {0.5, 2.5}}, ImageSize -> {60., 60.}}], {
         1.7339786718948398`, 1.9736705181885228`}], "DynamicName", BoxID -> 
        "VertexID$1"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.803921568627451, 0.5843137254901961, 0.4588235294117647], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{1., 0.}, {0., 2.}, {0., 0.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{0., 0.}, 0.25], 
                InsetBox["1", {0., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{0., 1.}, 0.25], 
                InsetBox["2", {0., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{0., 2.}, 0.25], 
                InsetBox["3", {0., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 0.}, 0.25], 
                InsetBox["4", {1., 0.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{2.7755575615628914`*^-17, 0.2833333333333332}, {
                2.7755575615628914`*^-17, 0.7166666666666667}}, {{
                0.2833333333333333, 0.}, {0.7166666666666668, 0.}}, {{
                2.7755575615628914`*^-17, 1.2833333333333332`}, {
                2.7755575615628914`*^-17, 1.7166666666666668`}}}]}}], {}}}, {
          PlotRange -> {{-0.5, 1.5}, {-0.5, 2.5}}, 
           ImageSize -> {60., 90.}}], {0.1588685363712495, 
         0.4025341331653798}], "DynamicName", BoxID -> "VertexID$2"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.9921568627450981, 0.8509803921568627, 0.7098039215686275], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{1., 0.}, {1., 1.}, {0., 2.}, {0., 1.}}], CurveClosed -> 
             True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{0., 1.}, 0.25], 
                InsetBox["2", {0., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{0., 2.}, 0.25], 
                InsetBox["3", {0., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 0.}, 0.25], 
                InsetBox["4", {1., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["5", {1., 1.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{2.7755575615628914`*^-17, 1.2833333333333332`}, {
                2.7755575615628914`*^-17, 1.7166666666666668`}}, {{
                0.2833333333333333, 1.}, {0.7166666666666668, 1.}}, {{1., 
                0.2833333333333332}, {1., 0.7166666666666667}}}]}}], {}}}, {
          PlotRange -> {{-0.5, 1.5}, {-0.5, 2.5}}, 
           ImageSize -> {60., 90.}}], {-0.024052427183611343`, 
         1.3856617594028307`}], "DynamicName", BoxID -> "VertexID$3"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.47058823529411764`, 0.8588235294117647, 0.8862745098039215], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{1., 0.}, {1., 2.}, {0., 2.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{0., 2.}, 0.25], 
                InsetBox["3", {0., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 0.}, 0.25], 
                InsetBox["4", {1., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["5", {1., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["6", {1., 2.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{0.2833333333333333, 2.}, {0.7166666666666668, 
                2.}}, {{1., 0.2833333333333332}, {1., 0.7166666666666667}}, {{
                1., 1.2833333333333332`}, {1., 
                1.7166666666666668`}}}]}}], {}}}, {
          PlotRange -> {{-0.5, 1.5}, {-0.5, 2.5}}, 
           ImageSize -> {60., 90.}}], {0.9740256436370539, 
         1.3236924959875633`}], "DynamicName", BoxID -> "VertexID$4"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[
            0.5294117647058824, 0.6627450980392157, 0.4196078431372549], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{2., 0.}, {1., 2.}, {1., 0.}}], CurveClosed -> True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{1., 0.}, 0.25], 
                InsetBox["4", {1., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["5", {1., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["6", {1., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 0.}, 0.25], 
                InsetBox["7", {2., 0.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{1., 0.2833333333333332}, {1., 
                0.7166666666666667}}, {{1.2833333333333332`, 0.}, {
                1.7166666666666668`, 0.}}, {{1., 1.2833333333333332`}, {1., 
                1.7166666666666668`}}}]}}], {}}}, {
          PlotRange -> {{0.5, 2.5}, {-0.5, 2.5}}, ImageSize -> {60., 90.}}], {
         1.1569466128675054`, 0.340564871583384}], "DynamicName", BoxID -> 
        "VertexID$5"], 
       TagBox[
        InsetBox[
         GraphicsBox[{{
            RGBColor[1., 0.6431372549019608, 0.4549019607843137], 
            AbsoluteThickness[15], 
            JoinForm["Round"], 
            JoinedCurveBox[
             Line[{{2., 0.}, {2., 1.}, {1., 2.}, {1., 1.}}], CurveClosed -> 
             True]}, {
            GraphicsGroupBox[{{
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[{
                 GrayLevel[0], 
                 Opacity[0.7]}], {
                GrayLevel[0], 
                CircleBox[{1., 1.}, 0.25], 
                InsetBox["5", {1., 1.}]}, {
                GrayLevel[0], 
                CircleBox[{1., 2.}, 0.25], 
                InsetBox["6", {1., 2.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 0.}, 0.25], 
                InsetBox["7", {2., 0.}]}, {
                GrayLevel[0], 
                CircleBox[{2., 1.}, 0.25], 
                InsetBox["8", {2., 1.}]}}, {
               Hue[0.6, 0.7, 0.5], 
               Opacity[0.7], 
               
               LineBox[{{{1., 1.2833333333333332`}, {1., 
                1.7166666666666668`}}, {{1.2833333333333332`, 1.}, {
                1.7166666666666668`, 1.}}, {{2., 0.2833333333333332}, {2., 
                0.7166666666666667}}}]}}], {}}}, {
          PlotRange -> {{0.5, 2.5}, {-0.5, 2.5}}, ImageSize -> {60., 90.}}], {
         1.916899629078195, 0.9905428883461539}], "DynamicName", BoxID -> 
        "VertexID$6"]}, {
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}}]}}]}, 
    DynamicBox[GraphComputation`NetworkGraphicsBox[
     1, Typeset`graph, Typeset`boxes], {
     CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
      SynchronousUpdating -> False, TrackedSymbols :> {}},
     ImageSizeCache->{{29.999999999999993`, 329.}, {-147.99999999999994`, 
      128.09319458943952`}}]]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None]], "Output",
 CellChangeTimes->{
  3.506456631011742*^9, 3.5065324083538637`*^9, 3.506535212171363*^9, 
   3.506738506594397*^9, 3.5067410732954197`*^9, {3.506788403480933*^9, 
   3.5067884338770437`*^9}, 3.5067885959059887`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cluster positioning", "Section",
 CellChangeTimes->{{3.5064868006604033`*^9, 3.506486809175617*^9}, {
  3.506531775729272*^9, 3.5065317760161943`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ycl", ",", "xcl"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"objCl", "[", "cluster_", "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"k", "=", 
      RowBox[{"idx", "[", 
       RowBox[{"jnodes", ",", "cluster"}], "]"}]}], ";", 
     RowBox[{"Total", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"cluster", "/.", "mcoords"}], ")"}], "-", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"xcl", "[", "k", "]"}], ",", 
           RowBox[{"ycl", "[", "k", "]"}]}], "}"}]}], ")"}], "^", "2"}], 
      "]"}]}], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"obj", "=", 
    RowBox[{"Total", "[", 
     RowBox[{"objCl", "/@", "jnodes"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"consCl", "[", 
     RowBox[{"{", 
      RowBox[{"bag1_", ",", "bag2_"}], "}"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"k1", "=", 
       RowBox[{"idx", "[", 
        RowBox[{"jnodes", ",", "bag1"}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"k2", "=", 
       RowBox[{"idx", "[", 
        RowBox[{"jnodes", ",", "bag2"}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"xcl", "[", "k1", "]"}], "-", 
           RowBox[{"xcl", "[", "k2", "]"}]}], ")"}], "^", "2"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"ycl", "[", "k1", "]"}], "-", 
           RowBox[{"ycl", "[", "k2", "]"}]}], ")"}], "^", "2"}]}], 
       "\[GreaterEqual]", "gap"}]}], "\[IndentingNewLine]", ")"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cons", "=", 
   RowBox[{"And", "@@", 
    RowBox[{"(", 
     RowBox[{"consCl", "/@", "jedges"}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"len", "=", 
   RowBox[{"Length", "@", "jnodes"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vars", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"xcl", "[", "#", "]"}], ",", 
        RowBox[{"ycl", "[", "#", "]"}]}], "}"}], "&"}], "/@", 
     RowBox[{"Range", "[", "len", "]"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.506486820066098*^9, 3.5064868372395678`*^9}, {
  3.506486882087532*^9, 3.506486894643169*^9}, {3.506486994136629*^9, 
  3.506487050340777*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"xcl", "[", "1", "]"}], ",", 
   RowBox[{"ycl", "[", "1", "]"}], ",", 
   RowBox[{"xcl", "[", "2", "]"}], ",", 
   RowBox[{"ycl", "[", "2", "]"}], ",", 
   RowBox[{"xcl", "[", "3", "]"}], ",", 
   RowBox[{"ycl", "[", "3", "]"}], ",", 
   RowBox[{"xcl", "[", "4", "]"}], ",", 
   RowBox[{"ycl", "[", "4", "]"}], ",", 
   RowBox[{"xcl", "[", "5", "]"}], ",", 
   RowBox[{"ycl", "[", "5", "]"}], ",", 
   RowBox[{"xcl", "[", "6", "]"}], ",", 
   RowBox[{"ycl", "[", "6", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.506487046054592*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "changes", " ", "graph", " ", "coordinates", " ", "so", " ", "that", " ", 
    "smallest", " ", "edge", " ", "has", " ", "length", " ", "minlen"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"rescaleGraph", "[", 
     RowBox[{"g_Graph", ",", "minlen_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"vcoords", "=", 
       RowBox[{"Thread", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"VertexList", "[", "g", "]"}], "->", "VertexCoordinates"}], 
         "/.", 
         RowBox[{"AbsoluteOptions", "[", "g", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"edgeLen", "[", 
        RowBox[{"a_", "\[UndirectedEdge]", "b_"}], "]"}], ":=", 
       RowBox[{"Norm", "[", 
        RowBox[{
         RowBox[{"a", "-", "b"}], "/.", "vcoords"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"edgeLen", "/@", 
       RowBox[{"EdgeList", "[", "g", "]"}]}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cluster global optimization", "Section",
 CellChangeTimes->{{3.506741032437076*^9, 3.506741056382066*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"gap", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"points", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0.`", ",", 
       RowBox[{"-", "0.314`"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.`", ",", "3.602`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.`", ",", 
       RowBox[{"-", "1.165`"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.`", ",", "0.78`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.272`", ",", "0.157`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "0.272`"}], ",", "0.157`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3.12`", ",", 
       RowBox[{"-", "1.801`"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "3.12`"}], ",", 
       RowBox[{"-", "1.801`"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.675`", ",", 
       RowBox[{"-", "0.39`"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "0.675`"}], ",", 
       RowBox[{"-", "0.39`"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1.009`", ",", "0.583`"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.009`"}], ",", "0.583`"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"n", "=", 
    RowBox[{"Length", "@", "points"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"objCl", "[", "k_", "]"}], ":=", 
   RowBox[{"Total", "[", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"points", "[", 
        RowBox[{"[", "k", "]"}], "]"}], "-", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"x", "[", "k", "]"}], ",", 
         RowBox[{"y", "[", "k", "]"}]}], "}"}]}], ")"}], "^", "2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"consCl", "[", 
    RowBox[{"j_", ",", "k_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"x", "[", "j", "]"}], "-", 
        RowBox[{"x", "[", "k", "]"}]}], ")"}], "^", "2"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"y", "[", "j", "]"}], "-", 
        RowBox[{"y", "[", "k", "]"}]}], ")"}], "^", "2"}]}], ">", "gap"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"varList", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"x", "[", "#", "]"}], ",", 
       RowBox[{"y", "[", "#", "]"}]}], "}"}], "&"}], "/@", 
    RowBox[{"Range", "[", "n", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"vars", "=", 
    RowBox[{"Flatten", "[", "varList", "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"obj", "=", 
   RowBox[{"Total", "[", 
    RowBox[{"objCl", "/@", 
     RowBox[{"Range", "[", "n", "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"cons", "=", 
    RowBox[{"Flatten", "@", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"consCl", "[", 
          RowBox[{"i", ",", "j"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", 
           RowBox[{"i", "+", "1"}], ",", "n"}], "}"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}]}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"Minimize", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"obj", ",", "cons"}], "}"}], ",", "vars"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"newPoints", "=", 
    RowBox[{"varList", "/.", 
     RowBox[{"Last", "@", "sol"}]}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"dist", "[", 
     RowBox[{"j_", ",", "k_", ",", "p_"}], "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"p", "[", 
         RowBox[{"[", 
          RowBox[{"j", ",", "1"}], "]"}], "]"}], "-", 
        RowBox[{"p", "[", 
         RowBox[{"[", 
          RowBox[{"k", ",", "1"}], "]"}], "]"}]}], ")"}], "^", "2"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"p", "[", 
         RowBox[{"[", 
          RowBox[{"j", ",", "2"}], "]"}], "]"}], "-", 
        RowBox[{"p", "[", 
         RowBox[{"[", 
          RowBox[{"k", ",", "2"}], "]"}], "]"}]}], ")"}], "^", "2"}]}]}], 
   ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"check", " ", "old", " ", "distances"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{"oldDists", "=", 
  RowBox[{"Sort", "@", 
   RowBox[{"Flatten", "@", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"dist", "[", 
         RowBox[{"i", ",", "j", ",", "points"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", 
          RowBox[{"i", "+", "1"}], ",", "n"}], "}"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "1", ",", 
        RowBox[{"n", "-", "1"}]}], "}"}]}], 
     "]"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"newDists", "=", 
   RowBox[{"Sort", "@", 
    RowBox[{"Flatten", "@", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"dist", "[", 
          RowBox[{"i", ",", "j", ",", "newPoints"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", 
           RowBox[{"i", "+", "1"}], ",", "n"}], "}"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}]}]}]}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"Plot", " ", "old", " ", "and", " ", "new", " ", "points"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{"Graphics", "[", 
  RowBox[{"MapThread", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Arrow", "[", 
      RowBox[{"{", 
       RowBox[{"#1", ",", "#2"}], "}"}], "]"}], "&"}], ",", 
    RowBox[{"{", 
     RowBox[{"points", ",", "newPoints"}], "}"}]}], "]"}], "]"}]}], "Input"]
}, Open  ]]
},
WindowSize->{740, 752},
WindowMargins->{{55, Automatic}, {-73, Automatic}},
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 6, \
2010)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 930, 25, 105, "Input"],
Cell[CellGroupData[{
Cell[1512, 49, 90, 1, 67, "Section"],
Cell[1605, 52, 5368, 150, 388, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7010, 207, 91, 1, 67, "Section"],
Cell[7104, 210, 2867, 73, 328, "Input"],
Cell[9974, 285, 12607, 308, 1183, "Input"],
Cell[CellGroupData[{
Cell[22606, 597, 400, 10, 43, "Input"],
Cell[23009, 609, 1946, 30, 128, "Message"],
Cell[24958, 641, 11061, 258, 367, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[36068, 905, 157, 2, 67, "Section"],
Cell[CellGroupData[{
Cell[36250, 911, 2623, 80, 223, "Input"],
Cell[38876, 993, 587, 15, 43, "Output"]
}, Open  ]],
Cell[39478, 1011, 1102, 30, 103, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[40617, 1046, 112, 1, 67, "Section"],
Cell[40732, 1049, 6174, 200, 373, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
