(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     55423,       1462]
NotebookOptionsPosition[     54547,       1427]
NotebookOutlinePosition[     54904,       1443]
CellTagsIndexPosition[     54861,       1440]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Bulatov`common`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Bulatov`dictionary`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Bulatov`treeDecomposition`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"On", "[", "Assert", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"$AssertFunction", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Message", "[", 
       RowBox[{
        RowBox[{"Assert", "::", "asrtf"}], ",", "#1"}], "]"}], ";", 
      RowBox[{"Print", "[", "#1", "]"}], ";", 
      RowBox[{"Abort", "[", "]"}]}], ")"}], "&"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.506387374989757*^9, 3.50638739632659*^9}, {
  3.506387631508575*^9, 3.5063876382754393`*^9}, {3.506389055679372*^9, 
  3.506389060200431*^9}}],

Cell[CellGroupData[{

Cell["Utils", "Section",
 CellChangeTimes->{{3.506368125503467*^9, 3.506368128017215*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"approxZero", "[", "x_", "]"}], ":=", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Chop", "[", "x", "]"}], "\[Equal]", "0"}], ")"}]}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"positive", " ", "list", " ", "of", " ", "numbers"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"isProbDist", "[", "dist_", "]"}], ":=", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"VectorQ", "[", "dist", "]"}], "&&", 
     RowBox[{"allTrue", "[", 
      RowBox[{
       RowBox[{"d", "\[Element]", "dist"}], ",", 
       RowBox[{"d", "\[GreaterEqual]", "0"}]}], "]"}], "&&", 
     RowBox[{"approxZero", "[", 
      RowBox[{
       RowBox[{"Total", "@", "dist"}], "-", "1"}], "]"}]}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"selectPositive", "[", "probs_", "]"}], ":=", 
   RowBox[{"Select", "[", 
    RowBox[{"probs", ",", "Positive"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"selectPositive", "[", 
     RowBox[{"probs_", ",", "stuff_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "poses", "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"poses", "=", 
        RowBox[{"Position", "[", 
         RowBox[{"probs", ",", 
          RowBox[{"_", "?", "Positive"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Extract", "[", 
          RowBox[{"#", ",", "poses"}], "]"}], "&"}], "/@", 
        RowBox[{"{", 
         RowBox[{"probs", ",", "stuff"}], "}"}]}]}]}], "\[IndentingNewLine]", 
     "]"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"entropy", ",", " ", 
    RowBox[{"ignore", " ", "0", " ", "probability", " ", "events"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"entropy", "[", "ys_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"isProbDist", "[", "ys", "]"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"relY", "=", 
       RowBox[{"selectPositive", "[", "ys", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"-", 
       RowBox[{"Total", "[", 
        RowBox[{"relY", " ", 
         RowBox[{"Log", "[", "relY", "]"}]}], "]"}]}]}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"evaluates", " ", "expected", " ", "energy"}], ",", " ", 
    RowBox[{"ignores", " ", "0", " ", "probability", " ", "events"}]}], " ", 
   "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"expectedEnergy", "[", 
     RowBox[{"probs_", ",", "pots_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"isProbDist", "[", "probs", "]"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"\[LeftBracketingBar]", "probs", "\[RightBracketingBar]"}], 
        "\[Equal]", 
        RowBox[{"\[LeftBracketingBar]", "pots", "\[RightBracketingBar]"}]}], 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"relProbs", ",", "relPots"}], "}"}], "=", 
       RowBox[{"selectPositive", "[", 
        RowBox[{"probs", ",", "pots"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Log", "[", "relPots", "]"}], ".", "relProbs"}]}], 
     "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "gets", " ", "all", " ", "pairs", " ", "of", " ", "sets", " ", "that", " ",
     "intersect"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"intersectingPairs", "[", "bags_List", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"cands", ",", "result"}], "}"}], ",", "\n", "\t", 
     RowBox[{
      RowBox[{"cands", "=", 
       RowBox[{
        RowBox[{"Outer", "[", 
         RowBox[{"List", ",", "bags", ",", "bags", ",", "1"}], "]"}], "//", 
        "fl1"}]}], ";", "\n", "\t", 
      RowBox[{"Select", "[", 
       RowBox[{"cands", ",", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "!=", 
           RowBox[{"#", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "&&", 
          RowBox[{"!", 
           RowBox[{"disjoint", "[", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "&&", 
          RowBox[{"OrderedQ", "[", "#", "]"}]}], "&"}]}], "]"}]}]}], "\n", 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.506368136396893*^9, 3.506368409627454*^9}, {
   3.506384887487013*^9, 3.506384892465054*^9}, {3.506390170281446*^9, 
   3.5063901706540203`*^9}, {3.506390219259161*^9, 3.506390224205965*^9}, 
   3.506390272155629*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Data structures", "Section",
 CellChangeTimes->{{3.506366039002769*^9, 3.506366047424727*^9}, 
   3.506366090556528*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "Messages", " ", "are", " ", "sparse", " ", "probability", " ", "tables", 
     " ", "with", " ", "list", " ", "of", " ", "x", " ", "values"}], ",", " ", 
    RowBox[{"list", " ", "of", " ", "y", " ", "values"}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{"isProbTable", "[", "table_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"\[LeftBracketingBar]", "table", "\[RightBracketingBar]"}], 
        "\[Equal]", "2"}], "&&", 
       RowBox[{
        RowBox[{"\[LeftBracketingBar]", 
         RowBox[{
         "table", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "\[RightBracketingBar]"}], "\[Equal]", 
        RowBox[{"\[LeftBracketingBar]", 
         RowBox[{
         "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
         "\[RightBracketingBar]"}]}], "&&", 
       RowBox[{"VectorQ", "[", 
        RowBox[{
        "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
        "]"}], "&&", 
       RowBox[{"allTrue", "[", 
        RowBox[{
         RowBox[{"k", "\[Element]", 
          RowBox[{
          "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
         ",", 
         RowBox[{"k", "\[GreaterEqual]", "0"}]}], "]"}], "&&", 
       RowBox[{"approxZero", "[", 
        RowBox[{
         RowBox[{"Total", "[", 
          RowBox[{
          "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
          "]"}], "-", "1"}], "]"}]}], "\[IndentingNewLine]", ")"}]}], ";"}], 
   "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"isProbTable", "[", "table_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"\[LeftBracketingBar]", "table", "\[RightBracketingBar]"}], 
        "\[Equal]", "2"}], "&&", 
       RowBox[{
        RowBox[{"\[LeftBracketingBar]", 
         RowBox[{
         "table", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "\[RightBracketingBar]"}], "\[Equal]", 
        RowBox[{"\[LeftBracketingBar]", 
         RowBox[{
         "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
         "\[RightBracketingBar]"}]}], "&&", 
       RowBox[{"VectorQ", "[", 
        RowBox[{
        "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
        "]"}], "&&", 
       RowBox[{"allTrue", "[", 
        RowBox[{
         RowBox[{"k", "\[Element]", 
          RowBox[{
          "table", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
         ",", 
         RowBox[{"k", "\[GreaterEqual]", "0"}]}], "]"}], "&&", 
       RowBox[{"OrderedQ", "[", 
        RowBox[{
        "table", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
        "]"}]}], "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]",
    "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"initialized", " ", "native", " ", "factor"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"isFactor", "[", "factor_", "]"}], ":=", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"Head", "[", "factor", "]"}], "===", "List"}], "&&", 
      RowBox[{"indexQ", "[", 
       RowBox[{"cX", ",", "factor"}], "]"}], "&&", 
      RowBox[{"indexQ", "[", 
       RowBox[{"cY", ",", "factor"}], "]"}], "&&", 
      RowBox[{"indexQ", "[", 
       RowBox[{"cD", ",", "factor"}], "]"}]}]}], ";"}], "\[IndentingNewLine]",
    "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"initialized", " ", "induced", " ", "factor"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"isMessage", "[", "msg_", "]"}], ":=", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"MatchQ", "[", 
       RowBox[{"msg", ",", 
        RowBox[{"{", 
         RowBox[{"_List", ",", "_List"}], "}"}]}], "]"}], "&&", 
      RowBox[{"MemberQ", "[", 
       RowBox[{"messages", ",", "msg"}], "]"}], "&&", 
      RowBox[{"indexQ", "[", 
       RowBox[{"cX", ",", "msg"}], "]"}], "&&", 
      RowBox[{"indexQ", "[", 
       RowBox[{"cY", ",", "msg"}], "]"}], "&&", 
      RowBox[{"indexQ", "[", 
       RowBox[{"cD", ",", "msg"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"isEvent", "[", "ev_", "]"}], ":=", 
     RowBox[{
      RowBox[{"Head", "[", 
       RowBox[{"ev", "\[Equal]", "List"}], "]"}], "&&", 
      RowBox[{"Depth", "[", 
       RowBox[{"ev", "\[Equal]", "2"}], "]"}]}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"table", " ", "event", " ", "list"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getXS", "[", "table_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"isProbTable", "[", "table", "]"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"First", "[", "table", "]"}]}], "\[IndentingNewLine]", ")"}]}],
     ";"}], "\[IndentingNewLine]", "\n", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"table", " ", "non"}], "-", 
     RowBox[{"zero", " ", "probabilities"}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getYS", "[", "table_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"isProbTable", "[", "table", "]"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Last", "[", "table", "]"}]}], "\[IndentingNewLine]", ")"}]}], 
    ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"normalizeTable", "[", "table_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"isProbTable", "@", "table"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xs", ",", "ys"}], "}"}], "=", "table"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"newYS", "=", 
        RowBox[{"Normalize", "[", 
         RowBox[{"ys", ",", "Total"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"xs", ",", "newYS"}], "}"}]}], "\[IndentingNewLine]", ")"}]}],
     ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"sortTable", "[", "table_", "]"}], ":=", 
     RowBox[{
      RowBox[{
       RowBox[{"table", "//", "Transpose"}], "//", "Sort"}], "//", 
      "Transpose"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"multiplyTables", "[", 
      RowBox[{"table1_", ",", "table2_"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{
         RowBox[{"isProbTable", "[", "table1", "]"}], "&&", 
         RowBox[{"isProbTable", "[", "table2", "]"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"sharedX", "=", 
        RowBox[{
         RowBox[{"getXS", "[", "table1", "]"}], "\[Intersection]", 
         RowBox[{"getXS", "[", "table2", "]"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"ys1", "=", 
        RowBox[{"Extract", "[", 
         RowBox[{
          RowBox[{"getYS", "@", "table1"}], ",", 
          RowBox[{"Position", "[", 
           RowBox[{
            RowBox[{"getXS", "@", "table1"}], ",", 
            RowBox[{"Alternatives", "@@", "sharedX"}]}], "]"}]}], "]"}]}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"ys2", "=", 
        RowBox[{"Extract", "[", 
         RowBox[{
          RowBox[{"getYS", "@", "table2"}], ",", 
          RowBox[{"Position", "[", 
           RowBox[{
            RowBox[{"getXS", "@", "table2"}], ",", 
            RowBox[{"Alternatives", "@@", "sharedX"}]}], "]"}]}], "]"}]}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"sharedX", ",", 
         RowBox[{"ys1", " ", "ys2"}]}], "}"}]}], "\[IndentingNewLine]", 
      ")"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"returns", " ", 
     RowBox[{"{", 
      RowBox[{"table1a", ",", "table2a", ",", "table1b", ",", 
       RowBox[{
       "table2b", " ", "where", " ", "a", " ", "tables", " ", "consists", " ",
         "of", " ", "shared", " ", "events"}], ",", " ", 
       RowBox[{
        RowBox[{"b", " ", "tables", " ", "consists", " ", "of", " ", "non"}], 
        "-", 
        RowBox[{"shared", " ", "events"}]}]}], " "}]}], "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"matchTables", "[", 
      RowBox[{"table1_", ",", "table2_"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xs1", ",", "xs2", ",", "ys1", ",", "ys2"}], "}"}], "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"getXS", "@", "table1"}], ",", 
          RowBox[{"getXS", "@", "table2"}], ",", 
          RowBox[{"getYS", "@", "table1"}], ",", 
          RowBox[{"getYS", "@", "table2"}]}], "}"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"sharedX", "=", 
        RowBox[{"xs1", "\[Intersection]", "xs2"}]}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"poses1All", ",", "poses2All"}], "}"}], "=", 
        RowBox[{
         RowBox[{
          RowBox[{"Position", "[", 
           RowBox[{"#", ",", 
            RowBox[{"_", "?", "isEvent"}]}], "]"}], "&"}], "/@", 
         RowBox[{"{", 
          RowBox[{"xs1", ",", "xs2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{
        RowBox[{"{", 
         RowBox[{"poses1a", ",", "poses2a"}], "}"}], "=", 
        RowBox[{
         RowBox[{
          RowBox[{"Position", "[", 
           RowBox[{"#", ",", 
            RowBox[{"Alternatives", "@@", "sharedX"}]}], "]"}], "&"}], "/@", 
         RowBox[{"{", 
          RowBox[{"xs1", ",", "xs2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"poses1b", "=", 
        RowBox[{"poses1All", "\[Backslash]", "poses1a"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"poses2b", "=", 
        RowBox[{"poses2All", "\[Backslash]", "poses2a"}]}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"table1a", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"Extract", "[", 
           RowBox[{"#", ",", "poses1a"}], "]"}], "&"}], "/@", 
         RowBox[{"{", 
          RowBox[{"xs1", ",", "ys1"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"table2a", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"Extract", "[", 
           RowBox[{"#", ",", "poses2a"}], "]"}], "&"}], "/@", 
         RowBox[{"{", 
          RowBox[{"xs2", ",", "ys2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"table1b", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"Extract", "[", 
           RowBox[{"#", ",", "poses1b"}], "]"}], "&"}], "/@", 
         RowBox[{"{", 
          RowBox[{"xs1", ",", "ys1"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"table2b", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"Extract", "[", 
           RowBox[{"#", ",", "poses2b"}], "]"}], "&"}], "/@", 
         RowBox[{"{", 
          RowBox[{"xs2", ",", "ys2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"table1a", ",", "table2a", ",", "table1b", ",", "table2b"}], 
        "}"}]}], "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]",
    "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"gives", " ", "full", " ", 
     RowBox[{"(", 
      RowBox[{"non", "-", "sparse"}], ")"}], " ", "representation", " ", "of",
      " ", "probability", " ", "table"}], " ", "*)"}], "\[IndentingNewLine]", 
   
   RowBox[{
    RowBox[{
     RowBox[{"tableToFull", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"xs_", ",", "ys_"}], "}"}], ",", "dom_"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"xsAll", "=", 
        RowBox[{"Subsets", "[", "dom", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"ysAll", "=", 
        RowBox[{"ys", "\[Tilde]", 
         RowBox[{"ConstantArray", "[", 
          RowBox[{"0", ",", 
           RowBox[{"\[LeftBracketingBar]", 
            RowBox[{"xsAll", "\[Backslash]", "xs"}], 
            "\[RightBracketingBar]"}]}], "]"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"sortTable", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"xs", "\[Tilde]", 
           RowBox[{"(", 
            RowBox[{"xsAll", "\[Backslash]", "xs"}], ")"}]}], ",", "ysAll"}], 
         "}"}], "]"}]}], "\[IndentingNewLine]", ")"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "total", " ", "variation", " ", "distance", " ", "between", " ", "two", 
     " ", "probability", " ", "tables"}], " ", "*)"}], "\[IndentingNewLine]", 
   
   RowBox[{
    RowBox[{
     RowBox[{"variationDistance", "[", 
      RowBox[{"t1_", ",", "t2_"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{
         RowBox[{"isProbTable", "[", "t1", "]"}], "&&", 
         RowBox[{"isProbTable", "[", "t2", "]"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"table1", ",", "table2"}], "}"}], "=", 
        RowBox[{"normalizeTable", "/@", 
         RowBox[{"{", 
          RowBox[{"t1", ",", "t2"}], "}"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"table1a", ",", "table2a", ",", "table1b", ",", "table2b"}], 
         "}"}], "=", 
        RowBox[{"matchTables", "[", 
         RowBox[{"table1", ",", "table2"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Total", "@", 
         RowBox[{"Abs", "[", 
          RowBox[{
           RowBox[{"getYS", "@", "table1a"}], "-", 
           RowBox[{"getYS", "@", "table2a"}]}], "]"}]}], "+", 
        RowBox[{"Total", "[", 
         RowBox[{"getYS", "@", "table1b"}], "]"}], "+", 
        RowBox[{"Total", "[", 
         RowBox[{"getYS", "@", "table2b"}], "]"}]}]}], "\[IndentingNewLine]", 
      ")"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"converts", " ", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2"}], "}"}], " ", "into", " ", "1"}], "&&", "2", "&&", 
     RowBox[{"\[Not]", "3"}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"eventToFormula", "[", 
      RowBox[{"event_", ",", "domain_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "negate", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Assert", "[", 
         RowBox[{"event", "\[Subset]", "domain"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"negate", "[", "lst_", "]"}], ":=", 
         RowBox[{"Not", "/@", "lst"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"And", "@@", 
         RowBox[{"(", 
          RowBox[{"event", "\[Tilde]", 
           RowBox[{"negate", "[", 
            RowBox[{"domain", "\[Backslash]", "event"}], "]"}]}], ")"}]}]}]}],
       "\n", "]"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "formula", " ", "representing", " ", "feasible", " ", "events", " ", "in",
      " ", "a", " ", "factor"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"factorEventsToFormula", "[", "factor_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"isFactor", "[", "factor", "]"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Or", "@@", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"eventToFormula", "[", 
            RowBox[{"#", ",", 
             RowBox[{"cD", "[", "factor", "]"}]}], "]"}], "&"}], "/@", 
          RowBox[{"cX", "[", "factor", "]"}]}], ")"}]}]}], "\n", ")"}]}], 
    ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "looks", " ", "probability", " ", "of", " ", "event", " ", "rawX", " ", 
     "according", " ", "to", " ", "given", " ", "factor"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"probLookup", "[", 
      RowBox[{"factor_", ",", "rawX_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "x", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Assert", "[", 
         RowBox[{"isFactor", "[", "factor", "]"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"x", "=", 
         RowBox[{"rawX", "\[Intersection]", 
          RowBox[{"cD", "[", "factor", "]"}]}]}], ";", "\[IndentingNewLine]", 
        
        RowBox[{"vals", "=", 
         RowBox[{"Extract", "[", 
          RowBox[{
           RowBox[{"cY", "[", "factor", "]"}], ",", 
           RowBox[{"Position", "[", 
            RowBox[{
             RowBox[{"cX", "[", "factor", "]"}], ",", "x"}], "]"}]}], "]"}]}],
         ";", "\[IndentingNewLine]", 
        RowBox[{"Assert", "[", 
         RowBox[{
          RowBox[{"Length", "[", "vals", "]"}], "==", "1"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"First", "@", "vals"}]}]}], "\n", "]"}]}], ";"}], 
   "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.506366075841078*^9, 3.5063660797685623`*^9}, {
   3.5063661390469303`*^9, 3.5063661456435347`*^9}, {3.506366183379257*^9, 
   3.506366254201736*^9}, {3.506366320270444*^9, 3.5063663321555357`*^9}, {
   3.5063663718479424`*^9, 3.5063664057253723`*^9}, {3.506366619301834*^9, 
   3.506366691813476*^9}, {3.506366881748991*^9, 3.506366934542561*^9}, {
   3.5063669802389603`*^9, 3.5063670877586946`*^9}, {3.506367143721133*^9, 
   3.506367175197566*^9}, {3.506367237772737*^9, 3.506367253439322*^9}, {
   3.506367626637652*^9, 3.50636777504642*^9}, {3.506367812859561*^9, 
   3.506367819402864*^9}, {3.506368940581291*^9, 3.506368951377747*^9}, {
   3.5063689898309193`*^9, 3.506368992093369*^9}, {3.5063690994019423`*^9, 
   3.506369111328423*^9}, {3.506381156725856*^9, 3.506381166995635*^9}, {
   3.506381273905987*^9, 3.5063812862405157`*^9}, {3.506381345717004*^9, 
   3.5063814712554493`*^9}, {3.5063821546539288`*^9, 3.506382217249642*^9}, {
   3.506382247319481*^9, 3.506382306302185*^9}, {3.506382372818446*^9, 
   3.5063824376873617`*^9}, 3.506382512539201*^9, {3.506382626847899*^9, 
   3.5063827132513733`*^9}, {3.506382763995175*^9, 3.506382966043536*^9}, {
   3.506383167253264*^9, 3.506383254808446*^9}, {3.50638329648179*^9, 
   3.506383299330977*^9}, {3.506383335437704*^9, 3.5063833813773108`*^9}, {
   3.506383487076539*^9, 3.506383606476774*^9}, {3.506383643543384*^9, 
   3.506383699313027*^9}, {3.5063837454657373`*^9, 3.506383858027321*^9}, {
   3.506383968647996*^9, 3.506383979613391*^9}, {3.506384079170753*^9, 
   3.506384105468522*^9}, {3.50638457250885*^9, 3.506384576747385*^9}, {
   3.506384641323956*^9, 3.506384754674128*^9}, {3.506387828194243*^9, 
   3.5063878312787523`*^9}, {3.506387875135818*^9, 3.5063878884820223`*^9}, {
   3.506389896777619*^9, 3.5063899334654922`*^9}, {3.5063900577235527`*^9, 
   3.5063900615375547`*^9}, 3.50639016304758*^9, {3.506390880153036*^9, 
   3.506390893366087*^9}, {3.506390946750704*^9, 3.50639096526225*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Setup", "Section",
 CellChangeTimes->{{3.506368098660709*^9, 3.506368101843402*^9}, {
  3.506368433904909*^9, 3.506368434461007*^9}}],

Cell["\<\
Global vars : nativeFactors, messages, g, verts, edges, vcoords, \
nativeFactors, jnodes, jedges\
\>", "Text",
 CellChangeTimes->{{3.506368583655373*^9, 3.506368619439027*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"factor", " ", "and", " ", "clique", " ", "children"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"ch", "[", 
      RowBox[{"{", "i_List", "}"}], "]"}], ":=", 
     RowBox[{"Cases", "[", 
      RowBox[{"messages", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "_"}], "}"}]}], "]"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"ch", "[", 
      RowBox[{"{", 
       RowBox[{"i_List", ",", "j_List"}], "}"}], "]"}], ":=", 
     RowBox[{"Cases", "[", 
      RowBox[{"messages", ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", 
         RowBox[{"Except", "[", "i", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"initializes", " ", "graph"}], ",", " ", 
     RowBox[{"and", " ", "graphical", " ", "model", " ", "factors"}]}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"initHardcore", "[", "graph_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{
         RowBox[{"Head", "[", "graph", "]"}], "===", "Graph"}], "]"}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"g", "=", "graph"}], ";", "\[IndentingNewLine]", 
       RowBox[{"verts", "=", 
        RowBox[{"Sort", "@", 
         RowBox[{"VertexList", "[", "g", "]"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"edges", "=", 
        RowBox[{"deepSort", "[", 
         RowBox[{"List", "@@@", 
          RowBox[{"EdgeList", "[", "g", "]"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"vcoords", "=", 
        RowBox[{"VertexCoordinates", "/.", 
         RowBox[{"AbsoluteOptions", "[", 
          RowBox[{"g", ",", "VertexCoordinates"}], "]"}]}]}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"nativeFactors", "=", 
        RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Clear", "[", 
        RowBox[{"cX", ",", "cY", ",", "cD"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"setupVertexFactor", "/@", "verts"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"setupEdgeFactor", "/@", "edges"}], ";"}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"setupVertexFactor", "[", "v_Integer", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{"nativeFactors", ",", 
         RowBox[{"{", "v", "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cD", "[", 
         RowBox[{"{", "v", "}"}], "]"}], "=", 
        RowBox[{"{", "v", "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cX", "[", 
         RowBox[{"{", "v", "}"}], "]"}], "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "v", "}"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cY", "[", 
         RowBox[{"{", "v", "}"}], "]"}], "=", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
      ")"}]}], ";"}], "\n", "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"setupEdgeFactor", "[", "ef_List", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{"nativeFactors", ",", "ef"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cD", "[", "ef", "]"}], "=", "ef"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cX", "[", "ef", "]"}], "=", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"First", "[", "ef", "]"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"Last", "[", "ef", "]"}], "}"}]}], "}"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cY", "[", "ef", "]"}], "=", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], ";"}], "\n", ")"}]}], 
    ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"initMessages", "[", 
      RowBox[{"{", 
       RowBox[{"dnodes_", ",", "dedges_"}], "}"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"jnodes", ",", "jedges"}], "}"}], "=", 
        RowBox[{"{", 
         RowBox[{"dnodes", ",", "dedges"}], "}"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"assignFactors", "[", "nativeFactors", "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"messages", "=", 
        RowBox[{"jedges", "\[Tilde]", 
         RowBox[{"(", 
          RowBox[{"Reverse", "/@", "jedges"}], ")"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"setupMessageFactor", "/@", "messages"}], ";"}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "greedily", " ", "assign", " ", "factors", " ", "to", " ", "regions"}], 
    " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"assignFactors", "[", "factors_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"assignToBag", "[", "jnode_", "]"}], ":=", 
        RowBox[{"(", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"coveredFactors", "=", 
           RowBox[{"Select", "[", 
            RowBox[{"unassigned", ",", 
             RowBox[{
              RowBox[{"#", "\[Subset]", "jnode"}], "&"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"cl", "[", "jnode", "]"}], "=", "coveredFactors"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"revCl", "[", "#", "]"}], "=", "jnode"}], ")"}], "&"}], 
           "/@", "coveredFactors"}], ";", "\[IndentingNewLine]", 
          RowBox[{"unassigned", "=", 
           RowBox[{"unassigned", "\[Backslash]", "coveredFactors"}]}], ";"}], 
         "\[IndentingNewLine]", ")"}]}], ";", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"Clear", "[", 
        RowBox[{"cl", ",", "revCl"}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"unassigned", "=", "factors"}], ";", "\[IndentingNewLine]", 
       RowBox[{"assignToBag", "/@", "jnodes"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Assert", "[", 
        RowBox[{"unassigned", "==", 
         RowBox[{"{", "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", ")"}]}], 
    ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "initialize", " ", "messages", " ", "to", " ", "constant", " ", 
     "factors"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"setupMessageFactor", "[", 
      RowBox[{"{", 
       RowBox[{"i_", ",", "j_"}], "}"}], "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"dom", "=", 
        RowBox[{"i", "\[Intersection]", "j"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cD", "[", 
         RowBox[{"{", 
          RowBox[{"i", ",", "j"}], "}"}], "]"}], "=", "dom"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cX", "[", 
         RowBox[{"{", 
          RowBox[{"i", ",", "j"}], "}"}], "]"}], "=", 
        RowBox[{"Subsets", "[", "dom", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"cY", "[", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j"}], "}"}], "]"}], "=", 
         RowBox[{"RandomReal", "[", 
          RowBox[{"1", ",", 
           RowBox[{"Length", "[", 
            RowBox[{"cX", "[", 
             RowBox[{"{", 
              RowBox[{"i", ",", "j"}], "}"}], "]"}], "]"}]}], "]"}]}], "*)"}],
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"cY", "[", 
         RowBox[{"{", 
          RowBox[{"i", ",", "j"}], "}"}], "]"}], "=", 
        RowBox[{"ConstantArray", "[", 
         RowBox[{"1.", ",", 
          RowBox[{"Length", "[", 
           RowBox[{"cX", "[", 
            RowBox[{"{", 
             RowBox[{"i", ",", "j"}], "}"}], "]"}], "]"}]}], "]"}]}]}], "\n", 
      ")"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.506368532658148*^9, 3.506368572514125*^9}, {
   3.5063850330892067`*^9, 3.5063850331746197`*^9}, {3.506387030199095*^9, 
   3.506387030749752*^9}, {3.506389191921319*^9, 3.50638922304455*^9}, 
   3.506800138828236*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Message Update", "Section",
 CellChangeTimes->{{3.506365792975038*^9, 3.5063657960409327`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"getMsg", "[", "msg_", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"isMessage", "[", "msg", "]"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"cX", "[", "msg", "]"}], ",", 
        RowBox[{"cY", "[", "msg", "]"}]}], "}"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"setMsg", "[", 
     RowBox[{"msg_", ",", 
      RowBox[{"{", 
       RowBox[{"xs_", ",", "ys_"}], "}"}]}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"isMessage", "[", "msg", "]"}], "&&", 
        RowBox[{"isProbTable", "[", 
         RowBox[{"{", 
          RowBox[{"xs", ",", "ys"}], "}"}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"cX", "[", "msg", "]"}], "=", "xs"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"cY", "[", "msg", "]"}], "=", "ys"}], ";"}], "\n", ")"}]}], 
   ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"RHS", " ", "of", " ", "message", " ", "equation"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"evalMsg", "[", 
     RowBox[{"{", 
      RowBox[{"i_", ",", "j_"}], "}"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{"isMessage", "[", 
        RowBox[{"{", 
         RowBox[{"i", ",", "j"}], "}"}], "]"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"factors", "=", 
       RowBox[{
        RowBox[{"cl", "[", "j", "]"}], "\[Tilde]", 
        RowBox[{"ch", "[", 
         RowBox[{"{", 
          RowBox[{"i", ",", "j"}], "}"}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"xsP", ",", "ysP"}], "}"}], "=", 
       RowBox[{"evalProduct", "[", 
        RowBox[{"factors", ",", "j"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"xsS", ",", "ysS"}], "}"}], "=", 
       RowBox[{"evalSum", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"xsP", ",", "ysP"}], "}"}], ",", 
         RowBox[{"i", "\[Intersection]", "j"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"normalizeTable", "[", 
       RowBox[{"{", 
        RowBox[{"xsS", ",", "ysS"}], "}"}], "]"}]}], "\n", ")"}]}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "evaluates", " ", "product", " ", "of", " ", "factors", " ", "on", " ", 
    "given", " ", "domain"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"evalProduct", "[", 
     RowBox[{"factors_", ",", "domain_"}], "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Assert", "[", 
       RowBox[{
        RowBox[{"Union", "[", 
         RowBox[{"Flatten", "[", 
          RowBox[{"cD", "/@", "factors"}], "]"}], "]"}], "\[Subset]", 
        "domain"}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "find", " ", "events", " ", "that", " ", "are", " ", "simultanously", 
        " ", "feasible", " ", "in", " ", "all", " ", "factors"}], " ", "*)"}],
       "\[IndentingNewLine]", 
      RowBox[{"formulas", "=", 
       RowBox[{"factorEventsToFormula", "/@", "factors"}]}], ";", 
      RowBox[{"instances", "=", 
       RowBox[{"SatisfiabilityInstances", "[", 
        RowBox[{
         RowBox[{"And", "@@", "formulas"}], ",", "domain", ",", "All"}], 
        "]"}]}], ";", 
      RowBox[{"xs", "=", 
       RowBox[{"Sort", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Extract", "[", 
           RowBox[{"domain", ",", 
            RowBox[{"Position", "[", 
             RowBox[{"#", ",", "True"}], "]"}]}], "]"}], "&"}], "/@", 
         "instances"}], "]"}]}], ";", 
      RowBox[{
       RowBox[{"oneVal", "[", "x_", "]"}], ":=", 
       RowBox[{"prod", "[", 
        RowBox[{
         RowBox[{"f", "\[Element]", "factors"}], ",", 
         RowBox[{"probLookup", "[", 
          RowBox[{"f", ",", "x"}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ys", "=", 
       RowBox[{"oneVal", "/@", "xs"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"xs", ",", "ys"}], "}"}]}], "\n", ")"}]}], ";"}], "\n", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "sums", " ", "over", " ", "atoms", " ", "orthogonal", " ", "to", " ", 
    "domain"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"evalSum", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "}"}], ",", 
       RowBox[{"{", "}"}]}], "}"}], ",", "_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "}"}]}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"evalSum", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"xs_", ",", "ys_"}], "}"}], ",", "dom_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"x", ",", "y"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{
         RowBox[{"isProbTable", "[", 
          RowBox[{"{", 
           RowBox[{"xs", ",", "ys"}], "}"}], "]"}], "&&", 
         RowBox[{"dom", "\[Subset]", 
          RowBox[{"Union", "[", 
           RowBox[{"Flatten", "[", "xs", "]"}], "]"}]}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"decorated", "=", 
        RowBox[{"Transpose", "[", 
         RowBox[{"{", 
          RowBox[{"xs", ",", "ys"}], "}"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"grouped", "=", 
        RowBox[{"GatherBy", "[", 
         RowBox[{"decorated", ",", 
          RowBox[{
           RowBox[{
            RowBox[{"First", "[", "#", "]"}], "\[Intersection]", "dom"}], 
           "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"getXY", "[", "group_", "]"}], ":=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"x", "=", 
           RowBox[{
            RowBox[{"group", "[", 
             RowBox[{"[", 
              RowBox[{"1", ",", "1"}], "]"}], "]"}], "\[Intersection]", 
            "dom"}]}], ";", 
          RowBox[{"y", "=", 
           RowBox[{"Total", "@", 
            RowBox[{"group", "[", 
             RowBox[{"[", 
              RowBox[{"All", ",", "2"}], "]"}], "]"}]}]}], ";", 
          RowBox[{"{", 
           RowBox[{"x", ",", "y"}], "}"}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Transpose", "[", 
        RowBox[{"getXY", "/@", "grouped"}], "]"}]}]}], "\n", "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"parallelUpdate", "[", "1", "]"}], ":=", 
    RowBox[{"(", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"oldTables", "=", 
       RowBox[{"getMsg", "/@", "messages"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"newTables", "=", 
       RowBox[{"evalMsg", "/@", "messages"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"changes", "=", 
       RowBox[{"MapThread", "[", 
        RowBox[{"variationDistance", ",", 
         RowBox[{"{", 
          RowBox[{"newTables", ",", "oldTables"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"MapThread", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"setMsg", "[", 
          RowBox[{"#1", ",", "#2"}], "]"}], "&"}], ",", 
        RowBox[{"{", 
         RowBox[{"messages", ",", "newTables"}], "}"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Total", "[", "changes", "]"}]}], "\[IndentingNewLine]", 
     ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"parallelUpdate", "[", "k_", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Assert", "[", 
     RowBox[{"k", "\[Element]", "Integers"}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"parallelUpdate", "[", "1", "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "1", ",", "k"}], "}"}]}], "]"}], ";"}], 
   "\[IndentingNewLine]", ")"}]}]}], "Input",
 CellChangeTimes->{{3.5063658246826563`*^9, 3.5063658249067593`*^9}, {
  3.506367323488542*^9, 3.506367330301176*^9}, {3.5063674719897747`*^9, 
  3.5063674773309526`*^9}, {3.506367518779955*^9, 3.50636762376402*^9}, {
  3.506367879371871*^9, 3.506367880843545*^9}, {3.506367957732423*^9, 
  3.506367968325349*^9}, {3.506368010674657*^9, 3.5063680703791933`*^9}, {
  3.506368753487937*^9, 3.5063687552929077`*^9}, {3.50638452753384*^9, 
  3.506384550841342*^9}, {3.5063848622152843`*^9, 3.506384874300791*^9}, {
  3.5063853527086554`*^9, 3.506385413855262*^9}, {3.506387989362814*^9, 
  3.50638799085629*^9}, {3.506388044827239*^9, 3.5063881035417013`*^9}, {
  3.506388133740037*^9, 3.5063881618813562`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Query", "Section",
 CellChangeTimes->{{3.506368719037643*^9, 3.506368721142171*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"evaluates", " ", "clique", " ", "using", " ", "current", " ", 
    RowBox[{"(", "LHS", ")"}], " ", "messages"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"evalClique", "[", "j_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"j", "\[Element]", "jnodes"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"factors", "=", 
        RowBox[{
         RowBox[{"cl", "[", "j", "]"}], "\[Tilde]", 
         RowBox[{"ch", "[", 
          RowBox[{"{", "j", "}"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xs", ",", "ys"}], "}"}], "=", 
        RowBox[{"normalizeTable", "@", 
         RowBox[{"evalProduct", "[", 
          RowBox[{"factors", ",", "j"}], "]"}]}]}]}], "\n", ")"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getMarginals", "[", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"occupationProb", "[", "table_", "]"}], ":=", 
        RowBox[{"Last", "@", 
         RowBox[{"getYS", "@", 
          RowBox[{"normalizeTable", "[", "table", "]"}]}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"getCliqueTable", "[", "vert_", "]"}], ":=", 
        RowBox[{"evalClique", "@", 
         RowBox[{"revCl", "[", 
          RowBox[{"{", "vert", "}"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"table", "[", 
        RowBox[{
         RowBox[{"v", "\[Element]", "verts"}], ",", 
         RowBox[{"occupationProb", "@", 
          RowBox[{"evalSum", "[", 
           RowBox[{
            RowBox[{"getCliqueTable", "[", "v", "]"}], ",", 
            RowBox[{"{", "v", "}"}]}], "]"}]}]}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"evaluates", " ", "separator", " ", "potential"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"evalEdge", "[", "jedge_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"msgFwd", "=", 
        RowBox[{"getMsg", "[", "jedge", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"msgBack", "=", 
        RowBox[{"getMsg", "[", 
         RowBox[{"Reverse", "@", "jedge"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"prodMsg", "=", 
        RowBox[{"multiplyTables", "[", 
         RowBox[{"msgFwd", ",", "msgBack"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"normalizeTable", "@", "prodMsg"}]}], "\[IndentingNewLine]", 
      ")"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "finds", " ", "discrepancy", " ", "on", " ", "separator", " ", 
     "marginal"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getMiscalibration", "[", "edge_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"edge", "\[Element]", "jedges"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"sep", "=", 
        RowBox[{"Intersection", "@@", "edge"}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"cliques", "=", 
        RowBox[{"Select", "[", 
         RowBox[{"jnodes", ",", 
          RowBox[{
           RowBox[{"sep", "\[Subset]", "#"}], "&"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"distrs", "=", 
        RowBox[{"evalClique", "/@", "edge"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"marg1", ",", "marg2"}], "}"}], "=", 
        RowBox[{
         RowBox[{
          RowBox[{"evalSum", "[", 
           RowBox[{"#", ",", "sep"}], "]"}], "&"}], "/@", "distrs"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"variationDistance", "[", 
        RowBox[{"marg1", ",", "marg2"}], "]"}]}], "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{"projDist", "[", 
        RowBox[{
         RowBox[{"getYS", "@", "marg1"}], ",", 
         RowBox[{"getYS", "@", "marg2"}]}], "]"}], "*)"}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getMaxMiscalibration", "[", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{"Max", "[", 
       RowBox[{"getMiscalibration", "/@", 
        RowBox[{"Select", "[", 
         RowBox[{"jedges", ",", "OrderedQ"}], "]"}]}], "]"}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getResidual", "[", "msg_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"msg", "\[Element]", "messages"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"oldMsg", "=", 
        RowBox[{"getMsg", "[", "msg", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"newMsg", "=", 
        RowBox[{"evalMsg", "[", "msg", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"variationDistance", "[", 
        RowBox[{"oldMsg", ",", "newMsg"}], "]"}]}], "\[IndentingNewLine]", 
      ")"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getTotalResidual", "[", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{"Total", "[", 
       RowBox[{"getResidual", "/@", "messages"}], "]"}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"cliqueAveEnergy", "[", "clique_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"clique", "\[Element]", "jnodes"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"factors", "=", 
        RowBox[{"cl", "[", "clique", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"cliquePot", "=", 
        RowBox[{"tableToFull", "[", 
         RowBox[{
          RowBox[{"evalProduct", "[", 
           RowBox[{"factors", ",", "clique"}], "]"}], ",", "clique"}], 
         "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"cliqueProb", "=", 
        RowBox[{"normalizeTable", "@", 
         RowBox[{"tableToFull", "[", 
          RowBox[{
           RowBox[{"evalClique", "[", "clique", "]"}], ",", "clique"}], 
          "]"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"expectedEnergy", "[", 
        RowBox[{
         RowBox[{"getYS", "@", "cliqueProb"}], ",", 
         RowBox[{"getYS", "@", "cliquePot"}]}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"cliqueEntropy", "[", "clique_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"clique", "\[Element]", "jnodes"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"entropy", "[", 
        RowBox[{"getYS", "@", 
         RowBox[{"evalClique", "@", "clique"}]}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"edgeEntropy", "[", "edge_", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Assert", "[", 
        RowBox[{"edge", "\[Element]", "jedges"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"entropy", "[", 
        RowBox[{"getYS", "@", 
         RowBox[{"evalEdge", "@", "edge"}]}], "]"}]}], "\[IndentingNewLine]", 
      ")"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"betheFreeEnergy", "[", "]"}], ":=", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Total", "[", 
        RowBox[{"cliqueAveEnergy", "/@", "jnodes"}], "]"}], "+", 
       RowBox[{"Total", "[", 
        RowBox[{"cliqueEntropy", "/@", "jnodes"}], "]"}], "-", 
       RowBox[{"Total", "[", 
        RowBox[{"edgeEntropy", "/@", "jedges"}], "]"}]}], 
      "\[IndentingNewLine]", ")"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.50636890649127*^9, 3.5063689357865143`*^9}, {
   3.50638401671945*^9, 3.506384018497675*^9}, {3.506384057246896*^9, 
   3.5063840574601517`*^9}, {3.506384242431795*^9, 3.506384244756207*^9}, {
   3.5063843510969143`*^9, 3.506384354105418*^9}, {3.506384443730638*^9, 
   3.506384489643169*^9}, {3.506385522788927*^9, 3.5063855408177767`*^9}, {
   3.5063860508309717`*^9, 3.50638616944386*^9}, {3.506386763379986*^9, 
   3.506386831636993*^9}, {3.5063868663118896`*^9, 3.5063868796612587`*^9}, {
   3.506386978793404*^9, 3.5063870060168753`*^9}, {3.506389141171795*^9, 
   3.506389141600412*^9}, {3.506389234578493*^9, 3.506389238567184*^9}, {
   3.5063894927621527`*^9, 3.506389493553125*^9}, 3.506389572484467*^9, {
   3.506389723329606*^9, 3.5063897491283493`*^9}, {3.5063897979221287`*^9, 
   3.506389846136778*^9}, {3.506390084302143*^9, 3.506390121481298*^9}, {
   3.506390332716228*^9, 3.506390337372121*^9}}]
}, Open  ]]
},
WindowSize->{740, 752},
WindowMargins->{{66, Automatic}, {-21, Automatic}},
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 6, \
2010)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 1029, 28, 103, "Input"],
Cell[CellGroupData[{
Cell[1611, 52, 90, 1, 67, "Section"],
Cell[1704, 55, 5198, 142, 493, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6939, 202, 126, 2, 67, "Section"],
Cell[7068, 206, 19721, 489, 1633, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[26826, 700, 139, 2, 67, "Section"],
Cell[26968, 704, 186, 4, 26, "Text"],
Cell[27157, 710, 8721, 227, 958, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[35915, 942, 101, 1, 67, "Section"],
Cell[36019, 945, 9138, 247, 823, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45194, 1197, 90, 1, 67, "Section"],
Cell[45287, 1200, 9244, 224, 1063, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
